""" XslTransformation API calls
"""
from core_main_app.commons import exceptions
from core_main_app.components.xsl_transformation.models import XslTransformation
from core_main_app.utils import xml
from core_main_app.utils.xml import is_well_formed_xml, has_xsl_namespace


def get_by_name(xslt_name):
    """ Get an XSLT document.

    Returns:
    """
    try:
        return XslTransformation.get_by_name(xslt_name)
    except:
        raise exceptions.ApiError("No transformation can be found with the given name")


def get_by_id(xslt_id):
    """ Get an XSLT document by its id.

    Args:
        xslt_id: Id.

    Returns:
        XslTransformation object.

    """
    return XslTransformation.get_by_id(xslt_id)


def get_all():
    """ Get list of XSLT document.

    Returns:
    """
    return XslTransformation.get_all()


def upsert(xsl_transformation):
    """ Upsert an xsl_transformation.

    Args:
        xsl_transformation: XslTransformation.

    Returns:
        XslTransformation instance.

    """
    if not is_well_formed_xml(xsl_transformation.content):
        raise exceptions.ApiError("Uploaded file is not well formatted XSLT.")
    elif not has_xsl_namespace(xsl_transformation.content):
        raise exceptions.ApiError("XSLT namespace not found in the uploaded file.")
    else:
        return xsl_transformation.save_object()


def delete(xsl_transformation):
    """ Delete an xsl_transformation.

    Args:
        xsl_transformation: XslTransformation to delete.

    """
    xsl_transformation.delete()


def xsl_transform(xml_content, xslt_name):
    """ Transform an XML file using an XSL transformation.

    Args:
        xml_content (str): XML document content, encoded in UTF-8
        xslt_name (str): Name of an XslTransformation document

    Returns:
        str: Transformed XML string
    """
    xslt_object = get_by_name(xslt_name)

    try:
        return xml.xsl_transform(xml_content, xslt_object.content)
    except Exception:
        raise exceptions.ApiError("An unexpected exception happened while transforming the XML")



""" Core Exceptions
"""


class CoreError(Exception):
    """ Exception raised by the Core
    """
    def __init__(self, message):
        self.message = message

    def __str__(self):
        return repr(self.message)


class ApiError(Exception):
    """ Exception raised by the API
    """
    def __init__(self, message):
        self.message = message

    def __str__(self):
        return repr(self.message)


class ModelError(Exception):
    """ Generic exception for the model
    """
    def __init__(self, message):
        self.message = message

    def __str__(self):
        return repr(self.message)


class RestApiError(Exception):
    """ Exception raised by the the REST Api
    """
    def __init__(self, message):
        self.message = message

    def __str__(self):
        return repr(self.message)


class DoesNotExist(Exception):
    """ Exception raised when an object does not exist
    """
    def __init__(self, message):
        self.message = message

    def __str__(self):
        return repr(self.message)


class NotUniqueError(Exception):
    """ Exception raised when an object is not unique
    """
    def __init__(self, message):
        self.message = message

    def __str__(self):
        return repr(self.message)


class XMLError(Exception):
    """ Exception raised by XML validation
    """
    def __init__(self, message):
        self.message = message

    def __str__(self):
        return repr(self.message)


class XSDError(Exception):
    """ Exception raised by XSD validation
    """
    def __init__(self, message):
        self.message = message

    def __str__(self):
        return repr(self.message)


class LockError(Exception):
    """ Exception raised when an object is locked
    """

    def __init__(self, message):
        self.message = message

    def __str__(self):
        return repr(self.message)



""" Common Validators
"""
from django.utils.translation import ungettext
from django.core.exceptions import ValidationError
from django.utils.translation import ugettext_lazy as _
from django.utils.deconstruct import deconstructible
from django.utils.encoding import force_text
from password_policies.forms.validators import BaseCountValidator
import os


@deconstructible
class BlankSpacesValidator(object):
    def __call__(self, value):
        value = force_text(value)
        if len(value.strip()) == 0:
            raise ValidationError(
                _('This field should not be empty.'),
            )


@deconstructible
class ExtensionValidator(object):
    def __init__(self, valid_extensions=list()):
        self.valid_extensions = valid_extensions

    def __call__(self, value):
        ext = os.path.splitext(value.name)[1]
        if not ext.lower() in self.valid_extensions:
            raise ValidationError(
                _('Unsupported file extension.'),
            )


class UpperCaseLetterCountValidator(BaseCountValidator):
    """ Counts the occurrences of letters and raises a :class:`~django.core.exceptions.ValidationError` if the count
    is less than :func:`~UpperCaseLetterCountValidator.get_min_count`.
    """
    categories = ['Lu']
    """ The unicode data letter categories:

    ====  ===========
    Code  Description
    ====  ===========
    LC    Letter, Cased
    Ll    Letter, Lowercase
    Lu    Letter, Uppercase
    Lt    Letter, Titlecase
    Lo    Letter, Other
    Nl    Number, Letter
    ====  ===========
    """
    #: The validator's error code.
    code = u"invalid_letter_count"

    def __init__(self, min_uppercase_letters):
        self.min_uppercase_letters = min_uppercase_letters

    def get_error_message(self):
        """ Returns this validator's error message.
        """
        msg = ungettext("The new password must contain %d or more uppercase letter.",
                        "The new password must contain %d or more uppercase letters.",
                        self.get_min_count()) % self.get_min_count()
        return msg

    def get_min_count(self):
        """ Returns: Min uppercase letters
        """
        return self.min_uppercase_letters


class LowerCaseLetterCountValidator(BaseCountValidator):
    """ Counts the occurrences of letters and raises a :class:`~django.core.exceptions.ValidationError` if the count
    is less than :func:`~LowerCaseLetterCountValidator.get_min_count`.
    """
    categories = ['Ll']
    """
    The unicode data letter categories:
    ====  ===========
    Code  Description
    ====  ===========
    LC    Letter, Cased
    Ll    Letter, Lowercase
    Lu    Letter, Uppercase
    Lt    Letter, Titlecase
    Lo    Letter, Other
    Nl    Number, Letter
    ====  ===========
    """
    #: The validator's error code.
    code = u"invalid_letter_count"

    def __init__(self, min_lowercase_letters):
        self.min_lowercase_letters = min_lowercase_letters

    def get_error_message(self):
        """ Returns this validator's error message.
        """
        msg = ungettext("The new password must contain %d or more lower letter.",
                        "The new password must contain %d or more lower letters.",
                        self.get_min_count()) % self.get_min_count()
        return msg

    def get_min_count(self):
        """ Returns: Min lowercase letters
        """
        return self.min_lowercase_letters



"""
    Common views
"""
from abc import ABCMeta

from django.core.urlresolvers import reverse
from django.http import HttpResponseBadRequest, HttpResponseForbidden
from django.http.response import HttpResponseRedirect
from django.utils.html import escape as html_escape
from django.views.generic import View

import core_main_app.components.data.api as data_api
from core_main_app.commons import exceptions
from core_main_app.commons.exceptions import DoesNotExist
from core_main_app.components.group import api as group_api
from core_main_app.components.template import api as template_api
from core_main_app.components.template_xsl_rendering import api as template_xsl_rendering_api
from core_main_app.components.version_manager import api as version_manager_api
from core_main_app.components.workspace import api as workspace_api
from core_main_app.components.xsl_transformation import api as xslt_transformation_api
from core_main_app.components.xsl_transformation.models import XslTransformation
from core_main_app.utils import group as group_utils
from core_main_app.utils.rendering import admin_render
from core_main_app.utils.rendering import render
from core_main_app.views.admin.forms import UploadXSLTForm, TemplateXsltRenderingForm
from core_main_app.utils.labels import get_data_label


class CommonView(View):
    """
        Abstract common view for admin and user.
    """
    __metaclass__ = ABCMeta

    administration = False

    def common_render(self, request, template_name, modals=None, assets=None, context=None):
        return admin_render(request, template_name, modals, assets, context) if self.administration \
            else render(request, template_name, modals, assets, context)


class EditWorkspaceRights(CommonView):
    """
        Edit workspace rights
    """

    template = "core_main_app/user/workspaces/edit_rights.html"

    def get(self, request, *args, **kwargs):

        try:
            workspace_id = kwargs['workspace_id']
            workspace = workspace_api.get_by_id(workspace_id)
        except DoesNotExist as e:
            return HttpResponseBadRequest("The workspace does not exist.")
        except:
            return HttpResponseBadRequest("Something wrong happened.")

        if workspace.owner != str(request.user.id) and not self.administration:
            return HttpResponseForbidden("Only the workspace owner can edit the rights.")

        try:
            # Users
            users_read_workspace = workspace_api.get_list_user_can_read_workspace(workspace, request.user)
            users_write_workspace = workspace_api.get_list_user_can_write_workspace(workspace, request.user)

            users_access_workspace = list(set(users_read_workspace + users_write_workspace))
            detailed_users = []
            for user in users_access_workspace:
                if str(user.id) != workspace.owner:
                    detailed_users.append({'object_id': user.id,
                                           'object_name': user.username,
                                           'can_read': user in users_read_workspace,
                                           'can_write': user in users_write_workspace,
                                           })
        except:
            detailed_users = []

        try:
            # Groups
            groups_read_workspace = workspace_api.get_list_group_can_read_workspace(workspace, request.user)
            groups_write_workspace = workspace_api.get_list_group_can_write_workspace(workspace, request.user)

            groups_access_workspace = list(set(groups_read_workspace + groups_write_workspace))
            group_utils.remove_list_object_from_list(groups_access_workspace,
                                                     [group_api.get_anonymous_group(), group_api.get_default_group()])
            detailed_groups = []
            for group in groups_access_workspace:
                detailed_groups.append({'object_id': group.id,
                                        'object_name': group.name,
                                        'can_read': group in groups_read_workspace,
                                        'can_write': group in groups_write_workspace,
                                        })
        except:
            detailed_groups = []

        context = {
            'workspace': workspace,
            'user_data': detailed_users,
            'group_data': detailed_groups,
            'template': "core_main_app/user/workspaces/list/edit_rights_table.html",
            'action_read': 'action_read',
            'action_write': 'action_write',
            'user': 'user',
            'group': 'group',
        }

        if workspace_api.is_workspace_public(workspace):
            context.update({'is_public': True})
        if workspace_api.is_workspace_global(workspace):
            context.update({'is_global': True})

        assets = {
            "css": ['core_main_app/libs/datatables/1.10.13/css/jquery.dataTables.css',
                    'core_main_app/libs/fSelect/css/fSelect.css',
                    'core_main_app/common/css/switch.css'],

            "js": [{
                    "path": 'core_main_app/libs/datatables/1.10.13/js/jquery.dataTables.js',
                    "is_raw": True
                },
                {
                    "path": "core_main_app/libs/fSelect/js/fSelect.js",
                    "is_raw": False
                },
                {
                    "path": 'core_main_app/common/js/backtoprevious.js',
                    "is_raw": True
                },
                {
                    "path": 'core_main_app/user/js/workspaces/tables.js',
                    "is_raw": True
                },
                {
                    "path": 'core_main_app/user/js/workspaces/add_user.js',
                    "is_raw": False
                },
                {
                    "path": 'core_main_app/user/js/workspaces/list/modals/switch_right.js',
                    "is_raw": False
                },
                {
                    "path": 'core_main_app/user/js/workspaces/list/modals/remove_rights.js',
                    "is_raw": False
                },
                {
                    "path": 'core_main_app/user/js/workspaces/add_group.js',
                    "is_raw": False
                },
                {
                    "path": 'core_main_app/user/js/workspaces/init.js',
                    "is_raw": False
                }]
        }

        modals = ["core_main_app/user/workspaces/list/modals/add_user.html",
                  "core_main_app/user/workspaces/list/modals/switch_right.html",
                  "core_main_app/user/workspaces/list/modals/remove_rights.html",
                  "core_main_app/user/workspaces/list/modals/add_group.html"]

        return self.common_render(request, self.template,
                                  context=context,
                                  assets=assets,
                                  modals=modals)


class ViewData(CommonView):
    """
        View detail data.
    """
    template = 'core_main_app/user/data/detail.html'

    def get(self, request, *args, **kwargs):
        data_id = request.GET['id']

        try:
            data = data_api.get_by_id(data_id, request.user)

            context = {
                'data': data
            }

            assets = {
                "js": [
                    {
                        "path": 'core_main_app/common/js/XMLTree.js',
                        "is_raw": False
                    },
                    {
                        "path": 'core_main_app/user/js/data/detail.js',
                        "is_raw": False
                    },
                ],
                "css": ["core_main_app/common/css/XMLTree.css"],
            }
            return self.common_render(request, self.template, context=context, assets=assets)
        except exceptions.DoesNotExist:
            error_message = 'Data not found'
        except exceptions.ModelError:
            error_message = 'Model error'
        except Exception, e:
            error_message = e.message

        return self.common_render(request, 'core_main_app/common/commons/error.html',
                                  context={"error": "Unable to access the requested " + get_data_label() + ": {}.".format(error_message)})


class XSLTView(View):
    """XSLT view.
    """

    @staticmethod
    def get(request, *args, **kwargs):
        modals = [
            "core_main_app/common/xslt/list/modals/edit.html",
            "core_main_app/common/xslt/list/modals/delete.html"
        ]

        assets = {
            "js": [
                {
                    "path": "core_main_app/common/js/xslt/list/modals/edit.js",
                    "is_raw": False
                },
                {
                    "path": "core_main_app/common/js/xslt/list/modals/delete.js",
                    "is_raw": False
                }
            ],
        }

        context = {
            'object_name': 'XSLT',
            "xslt": xslt_transformation_api.get_all(),
            "update_url": reverse('core_main_app_upload_xslt')
        }

        return render(request, "core_main_app/common/xslt/list.html", modals=modals, assets=assets,
                      context=context)


def read_xsd_file(xsd_file):
    """Return the content of the file uploaded using Django FileField.

    Returns:

    """
    # put the cursor at the beginning of the file
    xsd_file.seek(0)
    # read the content of the file
    return xsd_file.read()


class UploadXSLTView(View):
    """Upload XSLT view.
    """
    form_class = UploadXSLTForm
    template_name = 'core_main_app/common/xslt/upload.html'
    object_name = 'XSLT'

    def __init__(self, **kwargs):
        super(UploadXSLTView, self).__init__(**kwargs)
        self.context = {}
        self.context.update({'object_name': self.object_name})

    def get(self, request, *args, **kwargs):
        self.context.update({'upload_form': self.form_class()})
        return render(request, self.template_name, context=self.context)

    def post(self, request, *args, **kwargs):
        form = self.form_class(request.POST, request.FILES)
        self.context.update({'upload_form': form})

        if form.is_valid():
            return self._save_xslt(request)
        else:
            # Display error from the form
            return render(request, self.template_name, context=self.context)

    def _save_xslt(self, request):
        """Save an XSLT.

        Args:
            request: Request.

        """
        try:
            # get the XSLT name
            name = request.POST['name']
            # get the file from the form
            xsd_file = request.FILES['upload_file']
            # read the content of the file
            xsd_data = read_xsd_file(xsd_file)
            xslt = XslTransformation(name=name, filename=xsd_file.name, content=xsd_data)
            xslt_transformation_api.upsert(xslt)

            return HttpResponseRedirect(reverse("core_main_app_xslt"))
        except Exception, e:
            self.context.update({'errors': html_escape(e.message)})
            return render(request, 'core_main_app/common/xslt/upload.html', context=self.context)


class TemplateXSLRenderingView(View):
    """Template XSL rendering view.
    """
    rendering = render
    save_redirect = "core_main_app_manage_template_versions"
    back_to_url = "core_main_app_manage_template_versions"
    form_class = TemplateXsltRenderingForm
    template_name = "core_main_app/common/templates_xslt/main.html"
    context = {}
    assets = {}

    def get(self, request, *args, **kwargs):
        """ GET request. Create/Show the form for the configuration.

        Args:
            request:
            *args:
            **kwargs:

        Returns:

        """
        template_id = kwargs.pop('template_id')
        # Get the template
        template = template_api.get(template_id)
        # Get template information (version)
        version_manager = version_manager_api.get_from_version(template)
        version_number = version_manager_api.get_version_number(version_manager, template_id)
        try:
            # Get the existing configuration to build the form
            template_xsl_rendering = template_xsl_rendering_api.get_by_template_id(template_id)
            data = {'id': template_xsl_rendering.id, 'template': template.id,
                    'list_xslt': template_xsl_rendering.list_xslt.id if template_xsl_rendering.list_xslt else None,
                    'detail_xslt': template_xsl_rendering.detail_xslt.id if template_xsl_rendering.detail_xslt else None
                   }
        except (Exception, exceptions.DoesNotExist):
            # If no configuration, new form with pre-selected fields.
            data = {'template': template.id, 'list_xslt': None, 'detail_xslt': None}

        self.context = {
            'template_title': version_manager.title,
            'template_version': version_number,
            "form_template_xsl_rendering": self.form_class(data),
            'url_back_to': reverse(self.back_to_url, kwargs={'version_manager_id': version_manager.id})
        }

        return self.rendering(request, self.template_name, context=self.context, assets=self.assets)

    def post(self, request, *args, **kwargs):
        """ POST request. Try to save the configuration.

        Args:
            request:
            *args:
            **kwargs:

        Returns:

        """
        form = self.form_class(request.POST, request.FILES)
        self.context.update({'form_template_xsl_rendering': form})

        if form.is_valid():
            return self._save_template_xslt(request)
        else:
            # Display error from the form
            return self.rendering(request, self.template_name, context=self.context)

    def _save_template_xslt(self, request):
        """Save a template xslt rendering.

        Args:
            request: Request.

        """
        try:
            # Get the list xslt instance
            try:
                list_xslt = xslt_transformation_api.get_by_id(request.POST.get('list_xslt'))
            except (Exception, exceptions.DoesNotExist):
                list_xslt = None
            # Get the detail xslt instance
            try:
                detail_xslt = xslt_transformation_api.get_by_id(request.POST.get('detail_xslt'))
            except (Exception, exceptions.DoesNotExist):
                detail_xslt = None

            template_xsl_rendering_api.add_or_delete(template_xsl_rendering_id=request.POST.get('id'),
                                                     template_id=request.POST.get('template'),
                                                     list_xslt=list_xslt, detail_xslt=detail_xslt)

            template = template_api.get(request.POST.get('template'))
            # Get template information (version)
            version_manager = version_manager_api.get_from_version(template)
            return HttpResponseRedirect(reverse(self.save_redirect, args=[version_manager.id]))
        except Exception, e:
            self.context.update({'errors': html_escape(e.message)})
            return self.rendering(request, self.template_name, context=self.context)



import io
from django import template
from django.contrib.staticfiles import finders
from django.template.base import Template

register = template.Library()


@register.simple_tag(takes_context=True)
def include_static(context, path, encoding="utf-8"):
    file_path = finders.find(path)
    with io.open(file_path, "r", encoding=encoding) as f:
        string = f.read()

    t = Template(string)

    return t.render(context)



""" REST abstract views for the template version manager API
"""
from abc import ABCMeta, abstractmethod

from django.http import Http404
from rest_framework import status
from rest_framework.exceptions import ValidationError
from rest_framework.response import Response
from rest_framework.views import APIView

from core_main_app.commons import exceptions as exceptions
from core_main_app.commons.exceptions import NotUniqueError, ApiError, XSDError
from core_main_app.components.template import api as template_api
from core_main_app.components.version_manager import api as version_manager_api
from core_main_app.rest.template_version_manager.serializers import TemplateVersionManagerSerializer, \
    CreateTemplateSerializer
from core_main_app.rest.template_version_manager.utils import can_user_modify_template_version_manager
from core_main_app.utils.access_control.exceptions import AccessControlError
from core_main_app.utils.boolean import to_bool


class AbstractTemplateVersionManagerList(APIView):
    """ List template version managers.
    """

    __metaclass__ = ABCMeta

    @abstractmethod
    def get_template_version_managers(self):
        """ Return template version managers.

        Returns:

        """
        raise NotImplementedError("get_template_version_managers method is not implemented.")

    def get(self, request):
        """ Get template version managers.

        Query Params:
            title: title
            is_disabled: [True|False]

        Args:
            request:

        Returns:

        """
        try:
            # Get objects
            object_list = self.get_template_version_managers()

            # Apply filters
            title = self.request.query_params.get('title', None)
            if title is not None:
                object_list = object_list.filter(title=title)

            is_disabled = self.request.query_params.get('is_disabled', None)
            if is_disabled is not None:
                object_list = object_list.filter(is_disabled=to_bool(is_disabled))

            # Serialize object
            serializer = TemplateVersionManagerSerializer(object_list, many=True)
            return Response(serializer.data, status=status.HTTP_200_OK)
        except Exception as api_exception:
            content = {'message': api_exception.message}
            return Response(content, status=status.HTTP_500_INTERNAL_SERVER_ERROR)


class AbstractStatusTemplateVersion(APIView):
    """
    Set template version status.
    """

    __metaclass__ = ABCMeta

    def get_object(self, pk):
        """ Get template from db

        Args:
            pk:

        Returns:

        """
        try:
            template_object = template_api.get(pk)
            template_version_manager_object = version_manager_api.get_from_version(template_object)
            can_user_modify_template_version_manager(template_version_manager_object, self.request.user)
            return template_object
        except exceptions.DoesNotExist:
            raise Http404

    @abstractmethod
    def status_update(self, template_object):
        """ Perform an update of the object status.

        Returns:

        """
        raise NotImplementedError("status_update method is not implemented.")

    def patch(self, request, pk):
        """ Set status.

        Args:
            request:

        Returns:

        """
        try:
            # Get object
            template_object = self.get_object(pk)

            # Set current template
            self.status_update(template_object)

            return Response(status=status.HTTP_200_OK)
        except Http404:
            content = {'message': 'Template not found.'}
            return Response(content, status=status.HTTP_404_NOT_FOUND)
        except AccessControlError as access_error:
            content = {'message': access_error.message}
            return Response(content, status=status.HTTP_403_FORBIDDEN)
        except ValidationError as validation_exception:
            content = {'message': validation_exception.detail}
            return Response(content, status=status.HTTP_400_BAD_REQUEST)
        except ApiError as api_error:
            content = {'message': api_error.message}
            return Response(content, status=status.HTTP_400_BAD_REQUEST)
        except Exception as api_exception:
            content = {'message': api_exception.message}
            return Response(content, status=status.HTTP_500_INTERNAL_SERVER_ERROR)


class AbstractTemplateVersionManagerDetail(APIView):
    """
    Template Version Manager Detail.
    """

    __metaclass__ = ABCMeta

    def get_object(self, pk):
        """ Get template version manager from db

        Args:
            pk:

        Returns:

        """
        try:
            template_version_manager_object = version_manager_api.get(pk)
            can_user_modify_template_version_manager(template_version_manager_object, self.request.user)
            return template_version_manager_object
        except exceptions.DoesNotExist:
            raise Http404


class AbstractStatusTemplateVersionManager(AbstractTemplateVersionManagerDetail):
    """
    Set template version manager status.
    """

    __metaclass__ = ABCMeta

    @abstractmethod
    def status_update(self, template_version_manager_object):
        """ Perform an update of the object status.

        Returns:

        """
        raise NotImplementedError("status_update method is not implemented.")

    def patch(self, request, pk):
        """ Set status.

        Args:
            request:

        Returns:

        """
        try:
            # Get object
            template_version_manager_object = self.get_object(pk)

            # Set current template
            self.status_update(template_version_manager_object)

            return Response(status=status.HTTP_200_OK)
        except Http404:
            content = {'message': 'Template Version Manager not found.'}
            return Response(content, status=status.HTTP_404_NOT_FOUND)
        except AccessControlError as access_error:
            content = {'message': access_error.message}
            return Response(content, status=status.HTTP_403_FORBIDDEN)
        except ValidationError as validation_exception:
            content = {'message': validation_exception.detail}
            return Response(content, status=status.HTTP_400_BAD_REQUEST)
        except Exception as api_exception:
            content = {'message': api_exception.message}
            return Response(content, status=status.HTTP_500_INTERNAL_SERVER_ERROR)


class AbstractTemplateList(APIView):
    """
    Create a template.
    """

    __metaclass__ = ABCMeta

    def post(self, request):
        """ Create a template.

        POST data
        {
        "title": "title",
        "filename": "filename",
        "content": "<xs:schema xmlns:xs='http://www.w3.org/2001/XMLSchema'><xs:element name='root'/></xs:schema>"
        }

        Note: "dependencies"= json.dumps({"schemaLocation1": "id1" ,"schemaLocation2":"id2"})

        Args:
            request:

        Returns:

        """
        try:
            # Build serializers
            template_serializer = CreateTemplateSerializer(data=request.data)
            template_version_manager_serializer = TemplateVersionManagerSerializer(data=request.data)

            # Validate data
            template_serializer.is_valid(True)
            template_version_manager_serializer.is_valid(True)

            # Save data
            template_version_manager_object = template_version_manager_serializer.save(user=self.get_user())
            template_serializer.save(template_version_manager=template_version_manager_object)

            return Response(template_serializer.data, status=status.HTTP_201_CREATED)
        except ValidationError as validation_exception:
            content = {'message': validation_exception.detail}
            return Response(content, status=status.HTTP_400_BAD_REQUEST)
        except NotUniqueError:
            content = {'message': "A template with the same title already exists."}
            return Response(content, status=status.HTTP_400_BAD_REQUEST)
        except XSDError as xsd_error:
            content = {'message': "XSD Error: " + xsd_error.message}
            return Response(content, status=status.HTTP_400_BAD_REQUEST)
        except Exception as api_exception:
            content = {'message': api_exception.message}
            return Response(content, status=status.HTTP_500_INTERNAL_SERVER_ERROR)

    @abstractmethod
    def get_user(self):
        raise NotImplementedError("get_user method is not implemented.")



""" Integration Test for Workspace API
"""
from core_main_app.commons import exceptions
from core_main_app.components.workspace import api as workspace_api
from core_main_app.utils.integration_tests.integration_base_transaction_test_case import \
    MongoIntegrationTransactionTestCase
from tests.components.user.fixtures.fixtures import UserFixtures
from tests.components.workspace.fixtures.fixtures import WorkspaceFixtures

TITLE_1 = 'title 1'


class TestCreateWorkspace(MongoIntegrationTransactionTestCase):

    def test_create_one_workspace(self):
        # Context
        user1 = UserFixtures().create_user()
        # Act
        workspace = workspace_api.create_and_save(TITLE_1, user1.id)
        # Assert
        self.assertEqual(workspace.title, TITLE_1)
        self.assertEqual(workspace.owner, str(user1.id))

    def test_create_two_workspaces_same_name_same_user(self):
        # Context
        user1 = UserFixtures().create_user()
        # Act
        workspace_api.create_and_save(TITLE_1, user1.id)
        with self.assertRaises(exceptions.NotUniqueError):
            workspace_api.create_and_save(TITLE_1, user1.id)

    def test_create_two_workspaces_same_name_different_case_same_user(self):
        # Context
        user1 = UserFixtures().create_user()
        # Act
        workspace_api.create_and_save(TITLE_1, user1.id)
        with self.assertRaises(exceptions.NotUniqueError):
            workspace_api.create_and_save(TITLE_1.upper(), user1.id)

    def test_create_two_workspaces_same_name_different_user(self):
        # Context
        number_workspace = len(workspace_api.get_all())
        user1 = UserFixtures().create_user(username="user1")
        user2 = UserFixtures().create_user(username="user2")
        # Act
        workspace_api.create_and_save(TITLE_1, user1.id)
        workspace_api.create_and_save(TITLE_1, user2.id)
        # Assert
        self.assertEqual(number_workspace + 2, len(workspace_api.get_all()))

    def test_create_workspace_without_owner(self):
        # Act
        workspace = workspace_api.create_and_save(TITLE_1)
        # Assert
        self.assertEqual(workspace.title, TITLE_1)
        self.assertIsNone(workspace.owner)


class TestIsWorkspaceGlobal(MongoIntegrationTransactionTestCase):
    """ Test Is Workspace Global """

    def test_is_workspace_global_true(self):
        # Context
        workspace = WorkspaceFixtures.create_global_workspace(TITLE_1)
        # Act
        result = workspace_api.is_workspace_global(workspace)
        # Assert
        self.assertTrue(result)

    def test_is_workspace_global_false(self):
        # Context
        workspace = WorkspaceFixtures.create_workspace('1', TITLE_1)
        # Act
        result = workspace_api.is_workspace_global(workspace)
        # Assert
        self.assertFalse(result)


class TestGetGlobalWorkspace(MongoIntegrationTransactionTestCase):
    """ Test Get Global Workspace """

    def test_get_global_workspace(self):
        # Context
        workspace = WorkspaceFixtures.create_global_workspace(TITLE_1)
        # Act
        result = workspace_api.get_global_workspace()
        # Assert
        self.assertEquals(workspace, result)

    def test_get_global_workspace_multiple_workspace(self):
        # Context
        global_workspace = WorkspaceFixtures.create_global_workspace(TITLE_1)
        workspace = WorkspaceFixtures.create_workspace('1', TITLE_1)
        # Act
        result = workspace_api.get_global_workspace()
        # Assert
        self.assertEquals(global_workspace, result)

    def test_get_global_workspace_not_global(self):
        # Context
        WorkspaceFixtures.create_workspace('1', TITLE_1)
        # Act
        with self.assertRaises(exceptions.DoesNotExist):
            workspace_api.get_global_workspace()



""" Fixtures files for Blobs
"""
from core_main_app.components.blob.models import Blob
from core_main_app.utils.integration_tests.fixture_interface import FixtureInterface


class BlobFixtures(FixtureInterface):
    """ Blob fixtures
    """
    blob_1 = None
    blob_2 = None
    blob_3 = None
    blob_collection = None

    def insert_data(self):
        """ Insert a set of Blobs.

        Returns:

        """
        # Make a connexion with a mock database
        self.generate_blob_collection()

    def generate_blob_collection(self):
        """ Generate a Blob collection.

        Returns:

            user 1 -> blob1, blob2
            user 2 -> blob3

        """
        # NOTE: no real file to avoid using unsupported GridFS mock
        self.blob_1 = Blob(filename='blob1',
                           user_id='1',
                           handle='handle1').save()
        self.blob_2 = Blob(filename='blob2',
                           user_id='1',
                           handle='handle2').save()
        self.blob_3 = Blob(filename='blob3',
                           user_id='2',
                           handle='handle3').save()

        self.blob_collection = [self.blob_1,
                                self.blob_2,
                                self.blob_3]



"""Custom context processor
"""
from django.conf import settings


def domain_context_processor(request):
    return {
        "WEBSITE_ADMIN_COLOR": settings.WEBSITE_ADMIN_COLOR if hasattr(settings, "WEBSITE_ADMIN_COLOR") else 'black',
        "WEBSITE_SHORT_TITLE": settings.WEBSITE_SHORT_TITLE if hasattr(settings, "WEBSITE_SHORT_TITLE") else '',

        'CUSTOM_TITLE': settings.CUSTOM_TITLE if hasattr(settings, 'CUSTOM_TITLE') else '',
        'CUSTOM_ORGANIZATION': settings.CUSTOM_ORGANIZATION if hasattr(settings, 'CUSTOM_ORGANIZATION') else '',
        'CUSTOM_NAME': settings.CUSTOM_NAME if hasattr(settings, 'CUSTOM_NAME') else '',
        'CUSTOM_SUBTITLE': settings.CUSTOM_SUBTITLE if hasattr(settings, 'CUSTOM_SUBTITLE') else '',
        'CUSTOM_DATA': settings.CUSTOM_DATA if hasattr(settings, 'CUSTOM_DATA') else '',
        'CUSTOM_URL': settings.CUSTOM_URL if hasattr(settings, 'CUSTOM_URL') else '',
        'USE_EMAIL': settings.USE_EMAIL is True if hasattr(settings, 'USE_EMAIL') else False,

        'DATA_SOURCES_EXPLORE_APPS': settings.DATA_SOURCES_EXPLORE_APPS if
        hasattr(settings, 'DATA_SOURCES_EXPLORE_APPS') else [],

        'INSTALLED_APPS': settings.INSTALLED_APPS if hasattr(settings, 'INSTALLED_APPS') else [],
    }



"""Query builder class
"""
from bson.objectid import ObjectId

from core_main_app.utils.query.constants import VISIBILITY_PUBLIC, VISIBILITY_ALL, VISIBILITY_USER
from core_main_app.utils.query.mongo.prepare import prepare_query
import json
from core_main_app.components.workspace import api as workspace_api


class QueryBuilder(object):
    """Query builder class
    """

    def __init__(self, query, sub_document_root):
        """Creates query builder

        Args:
            query:
            sub_document_root:
        """
        self.criteria = [prepare_query(json.loads(query),
                                       regex=True,
                                       sub_document_root=sub_document_root)]

    def add_list_templates_criteria(self, list_template_ids):
        """Adds a criteria on template ids

        Args:
            list_template_ids:

        Returns:

        """
        self.criteria.append({'template': {'$in': [ObjectId(template_id) for template_id in list_template_ids]}})

    def add_visibility_criteria(self, visibility):
        """Adds a criteria on visibility

        Args:
            visibility:

        Returns:

        """
        if visibility == VISIBILITY_PUBLIC:
            self.criteria.append({'workspace':
                                      {'$in': [ObjectId(workspace_id)
                                               for workspace_id
                                               in workspace_api.get_all_public_workspaces().values_list('id')]}})
        elif visibility == VISIBILITY_ALL:
            # NOTE: get all data, no restriction needed
            pass
        elif visibility == VISIBILITY_USER:
            # TODO: get only user data
            pass

    def get_raw_query(self):
        """Returns the raw query

        Returns:

        """
        # create a raw query
        if len(self.criteria) > 1:
            # more than one criteria, create a AND query
            raw_query = {"$and": self.criteria}
        else:
            # one criteria, raw query is the criteria
            raw_query = self.criteria[0]

        return raw_query



""" Core main app user views
"""
from django.conf import settings
from django.contrib.auth import authenticate, login, logout
from django.contrib.auth.decorators import login_required
from django.contrib.staticfiles import finders
from django.core.urlresolvers import reverse
from django.http.response import HttpResponse
from django.shortcuts import redirect
from rest_framework.status import HTTP_405_METHOD_NOT_ALLOWED

from core_main_app.components.version_manager import api as version_manager_api
from core_main_app.settings import INSTALLED_APPS
from core_main_app.utils.rendering import render
from core_main_app.views.user.forms import LoginForm


def custom_login(request):
    """ Custom login page.
    
        Parameters:
            request: 
    
        Returns:
    """
    def _login_redirect(to_page):
        if to_page is not None and to_page != "":
            return redirect(to_page)

        return redirect(reverse("core_main_app_homepage"))

    if request.method == "POST":
        username = request.POST["username"]
        password = request.POST["password"]
        next_page = request.POST["next_page"]

        try:
            user = authenticate(username=username, password=password)

            if not user.is_active:
                return render(request, "core_main_app/user/login.html",
                              context={'login_form': LoginForm(initial={'next_page': next_page}), 'login_locked': True,
                                       'with_website_features': "core_website_app" in INSTALLED_APPS})

            login(request, user)

            return _login_redirect(next_page)
        except Exception as e:
            return render(request, "core_main_app/user/login.html",
                          context={'login_form': LoginForm(initial={'next_page': next_page}), 'login_error': True,
                                   'with_website_features': "core_website_app" in INSTALLED_APPS})
    elif request.method == "GET":
        if request.user.is_authenticated():
            return redirect(reverse("core_main_app_homepage"))

        next_page = None
        if "next" in request.GET:
            next_page = request.GET["next"]

        return render(request, "core_main_app/user/login.html",
                      context={'login_form': LoginForm(initial={"next_page": next_page}),
                               'with_website_features': "core_website_app" in INSTALLED_APPS})
    else:
        return HttpResponse(status=HTTP_405_METHOD_NOT_ALLOWED)


def custom_logout(request):
    """ Custom logout page.
    
        Parameters:
            request: 
    
        Returns:
    """
    logout(request)
    return redirect(reverse("core_main_app_login"))


def homepage(request):
    """ Homepage for the website

        Parameters:
            request:

        Returns:
    """
    assets = {
        "js": []
    }

    if finders.find("core_main_app/css/homepage.css") is not None:
        assets["css"] = ["core_main_app/css/homepage.css"]

    if finders.find("core_main_app/js/homepage.js") is not None:
        assets["js"].append(
            {
                "path": "core_main_app/js/homepage.js",
                "is_raw": False
            }
        )

    return render(request, "core_main_app/user/homepage.html", assets=assets)


@login_required(login_url='/login')
def manage_template_versions(request, version_manager_id):
    """View that allows template versions management

    Args:
        request:
        version_manager_id:

    Returns:

    """
    try:
        # get the version manager
        version_manager = version_manager_api.get(version_manager_id)

        if not request.user.is_staff and version_manager.user != str(request.user.id):
            raise Exception("You don't have the rights to perform this action.")

        context = get_context_manage_template_versions(version_manager)
        if 'core_parser_app' in settings.INSTALLED_APPS:
            context.update({"module_url": "core_parser_app_template_modules"})

        assets = {
                    "js": [
                        {
                            "path": 'core_main_app/common/js/templates/versions/set_current.js',
                            "is_raw": False
                        },
                        {
                            "path": 'core_main_app/common/js/templates/versions/restore.js',
                            "is_raw": False
                        },
                        {
                            "path": 'core_main_app/common/js/templates/versions/modals/disable.js',
                            "is_raw": False
                        }
                    ]
                }

        modals = ["core_main_app/admin/templates/versions/modals/disable.html"]

        return render(request,
                      'core_main_app/common/templates/versions.html',
                      assets=assets,
                      modals=modals,
                      context=context)
    except Exception, e:
        return render(request,
                      'core_main_app/common/commons/error.html',
                      context={'error': e.message})


def get_context_manage_template_versions(version_manager):
    """ Get the context to manage the template versions.

    Args:
        version_manager:

    Returns:

    """

    # Use categorized version for easier manipulation in template
    versions = version_manager.versions
    categorized_versions = {
        "available": [],
        "disabled": []
    }
    for index, version in enumerate(versions, 1):
        indexed_version = {
            "index": index,
            "object": version
        }

        if version not in version_manager.disabled_versions:
            categorized_versions["available"].append(indexed_version)
        else:
            categorized_versions["disabled"].append(indexed_version)

    version_manager.versions = categorized_versions
    context = {
        "object_name": "Template",
        "version_manager": version_manager
    }

    return context



""" REST abstract views for the data API
"""
import json
from abc import ABCMeta, abstractmethod

from rest_framework import status
from rest_framework.response import Response
from rest_framework.views import APIView

from core_main_app.components.data import api as data_api
from core_main_app.utils.query.constants import VISIBILITY_OPTION
from core_main_app.utils.query.mongo.query_builder import QueryBuilder


class AbstractExecuteLocalQueryView(APIView):
    sub_document_root = 'dict_content'

    __metaclass__ = ABCMeta

    def get(self, request):
        """Execute query on local instance and return results

        Args:
            request:

        Returns:

        """

        return self.execute_query()

    def post(self, request):
        """Execute query on local instance and return results

        Warning:
            Need to backslash double quotes in JSON payload.

        Example:
            {"query": "{\"$or\": [{\"image.owner\": \"Peter\"}, {\"image.owner.#text\":\"Peter\"}]}"}

        Args:
            request:

        Returns:

        """

        return self.execute_query()

    def execute_query(self):
        """ Compute and return query results.
        """
        try:
            # get query and templates
            query = self.request.data.get('query', None)
            templates = json.loads(self.request.data.get('templates', '[]'))
            options = json.loads(self.request.data.get('options', '{}'))

            if query is not None:
                # prepare query
                raw_query = self.build_query(query, templates, options)
                # execute query
                data_list = self.execute_raw_query(raw_query)
                # build and return response
                return self.build_response(data_list)
            else:
                content = {'message': 'Expected parameters not provided.'}
                return Response(content, status=status.HTTP_400_BAD_REQUEST)
        except Exception as api_exception:
            content = {'message': api_exception.message}
            return Response(content, status=status.HTTP_500_INTERNAL_SERVER_ERROR)

    def build_query(self, query, templates,  options):
        """ Build the raw query.
        Args:
            query:
            templates:
            options:

        Returns:
            The raw query.

        """
        # build query builder
        query_builder = QueryBuilder(query, self.sub_document_root)
        # update the criteria with templates information
        if len(templates) > 0:
            list_template_ids = [template['id'] for template in templates]
            query_builder.add_list_templates_criteria(list_template_ids)
        if VISIBILITY_OPTION in options:
            query_builder.add_visibility_criteria(options[VISIBILITY_OPTION])

        # get raw query
        return query_builder.get_raw_query()

    def execute_raw_query(self, raw_query):
        """ Execute the raw query in database.
        Args:
            raw_query: Query to execute.

        Returns:
            Results of the query.

        """
        return data_api.execute_query(raw_query, self.request.user)

    @abstractmethod
    def build_response(self, data_list):
        """ Build the paginated response.

        Args:
            data_list: List of data.

        Returns:
            The response.

        """
        raise NotImplementedError("build_response method is not implemented.")



"""File utils
"""
from io import BytesIO
from mimetypes import guess_type

from core_main_app.commons.exceptions import CoreError
from django.http.response import HttpResponse


def get_file_http_response(file_content, file_name, content_type=None, extension=''):
    """Return http response with file to download.

    Args:
        file_content:
        file_name:
        content_type:
        extension:

    Returns:

    """
    try:
        # set file content
        try:
            _file = BytesIO(file_content.encode('utf-8'))
        except Exception:
            _file = BytesIO(file_content)
        # guess file content type if not set
        if content_type is None:
            content_type = guess_type(file_name)
        # set file in http response
        response = HttpResponse(_file, content_type=content_type)
        # set filename extension
        if not file_name.endswith(extension):
            if not extension.startswith("."):
                extension = "." + extension
            file_name += extension
        # set content disposition in response
        response['Content-Disposition'] = 'attachment; filename=' + file_name
        # return response
        return response
    except Exception:
        raise CoreError('An unexpected error occurred.')


def read_file_content(file_path):
    """Read the content of a file.

    Args:
        file_path:

    Returns:

    """
    with open(file_path) as _file:
        file_content = _file.read()
        return file_content



"""Unit tests for template version manager rest api
"""
from django.test import SimpleTestCase
from mock.mock import patch
from rest_framework import status

from core_main_app.commons.exceptions import DoesNotExist
from core_main_app.components.template_version_manager.models import TemplateVersionManager
from core_main_app.components.version_manager.models import VersionManager
from core_main_app.rest.template_version_manager import views
from core_main_app.utils.tests_tools.MockUser import create_mock_user
from core_main_app.utils.tests_tools.RequestMock import RequestMock


class TestGlobalTemplateVersionManagerList(SimpleTestCase):
    def setUp(self):
        super(TestGlobalTemplateVersionManagerList, self).setUp()

    @patch.object(TemplateVersionManager, 'get_all')
    def test_get_all_returns_http_200(self, mock_get_all):
        # Arrange
        mock_user = create_mock_user('1')
        mock_get_all.return_value = []

        # Mock
        response = RequestMock.do_request_get(views.GlobalTemplateVersionManagerList.as_view(),
                                              mock_user)

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)


class TestUserTemplateVersionManagerList(SimpleTestCase):
    def setUp(self):
        super(TestUserTemplateVersionManagerList, self).setUp()

    @patch.object(TemplateVersionManager, 'get_all')
    def test_get_all_returns_http_200(self, mock_get_all):
        # Arrange
        mock_user = create_mock_user('1')
        mock_get_all.return_value = []

        # Mock
        response = RequestMock.do_request_get(views.UserTemplateVersionManagerList.as_view(),
                                              mock_user)

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)


class TestTemplateVersionManagerDetail(SimpleTestCase):
    def setUp(self):
        super(TestTemplateVersionManagerDetail, self).setUp()

    @patch.object(VersionManager, 'get_by_id')
    def test_get_returns_http_200_when_data_exists(self, mock_get_by_id):
        # Arrange
        mock_user = create_mock_user('1')
        mock_get_by_id.return_value = TemplateVersionManager()
        # Mock
        response = RequestMock.do_request_get(views.TemplateVersionManagerDetail.as_view(),
                                              mock_user,
                                              param={'pk': 'id'})

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)

    @patch.object(VersionManager, 'get_by_id')
    def test_get_returns_http_404_when_data_not_found(self, mock_get_by_id):
        # Arrange
        mock_user = create_mock_user('1')
        mock_get_by_id.side_effect = DoesNotExist("error")

        # Mock
        response = RequestMock.do_request_get(views.TemplateVersionManagerDetail.as_view(),
                                              mock_user,
                                              param={'pk': 'invalid'})

        # Assert
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)



""" Apps file for setting core package when app is ready
"""
from django.apps import AppConfig

import core_main_app.permissions.discover as discover
from core_main_app.components.data.models import Data
from core_main_app.utils.databases.mongoengine_database import init_text_index


class InitApp(AppConfig):
    """ Core application settings
    """
    name = 'core_main_app'

    def ready(self):
        """ Run when the app is ready.

        Returns:

        """
        discover.init_rules(self.apps)
        discover.create_public_workspace()
        init_text_index(Data)



from django.conf.urls import url, include
from django.contrib import admin
from core_main_app import urls as core_main_app_urls

urlpatterns = [
    url(r'^admin/', include(admin.site.urls)),
] + core_main_app_urls.urlpatterns



"""Mailing task
"""
from __future__ import absolute_import

from logging import getLogger

from celery import shared_task
from django.core.mail import send_mail as django_send_mail, mail_admins, mail_managers, BadHeaderError
from django.template import loader, Context

from core_main_app.settings import SERVER_EMAIL, EMAIL_SUBJECT_PREFIX

logger = getLogger(__name__)


@shared_task
def send_mail(recipient_list, subject, path_to_template, context={}, fail_silently=True, sender=SERVER_EMAIL):
    """Send email.

    Args:
        recipient_list:
        subject:
        path_to_template:
        context:
        fail_silently:
        sender:

    Returns:

    """
    try:
        # Render the given template with context information
        template = loader.get_template(path_to_template)
        context = Context(context)
        message = template.render(context)
        # Send mail
        django_send_mail(subject=EMAIL_SUBJECT_PREFIX+subject, message='', from_email=sender,
                         recipient_list=recipient_list,
                         html_message=message, fail_silently=fail_silently)
    except BadHeaderError, e:
        raise e
    except Exception, e:
        raise e


@shared_task
def send_mail_to_administrators(subject, path_to_template, context={}, fail_silently=True):
    """Send email to administrators.

    Args:
        subject:
        path_to_template:
        context:
        fail_silently:

    Returns:

    """
    try:
        # Render the given template with context information
        template = loader.get_template(path_to_template)
        context = Context(context)
        message = template.render(context)
        # Send mail
        mail_admins(subject=subject, message='', html_message=message, fail_silently=fail_silently)
    except BadHeaderError:
        pass
    except Exception:
        pass


@shared_task
def send_mail_to_managers(subject, path_to_template, context={}, fail_silently=True):
    """Send email to managers.

    Args:
        subject:
        path_to_template:
        context:
        fail_silently:

    Returns:

    """
    try:
        # Render the given template with context information
        template = loader.get_template(path_to_template)
        context = Context(context)
        message = template.render(context)
        # Send mail
        mail_managers(subject=subject, message='', html_message=message, fail_silently=fail_silently)
    except BadHeaderError:
        pass
    except Exception:
        pass




"""Pagination utils for rest_framework package
"""
from urlparse import urlparse
from core_explore_common_app.commons.exceptions import PaginationError
from core_main_app.utils.pagination.rest_framework_paginator.pagination import StandardResultsSetPagination


def get_page_number(url):
    """Get page number from url

    Args:
        url:

    Returns:

    """

    if url is None:
        return None

    try:
        # parse string url
        object_url = urlparse(url)
        # get query from url
        query_url = object_url.query
        # rest framework returns base (without 'page=') url when page is 1
        if "page=" not in query_url:
            return 1
        else:
            return query_url.split('=')[1]
    except Exception, e:
        return PaginationError("An error occurred when getting page number from url: {}.".format(e.message))


def get_paginator():
    """Create a paginator

    Returns:

    """
    # return paginator
    return StandardResultsSetPagination()



""" App model to manage permissions.
"""
from django.db import models

from core_main_app.permissions import rights
from core_main_app.permissions.utils import get_formatted_name


class Main(models.Model):
    class Meta:
        verbose_name = 'core_main_app'
        default_permissions = ()
        permissions = (
            (rights.publish_data, get_formatted_name(rights.publish_data)),
        )



"""Serializers used throughout the workspace Rest API
"""
from rest_framework_mongoengine.serializers import DocumentSerializer
from core_main_app.components.workspace.models import Workspace
from core_main_app.components.workspace import api as workspace_api


class WorkspaceSerializer(DocumentSerializer):
    """ Workspace serializer
    """
    class Meta:
        """ Meta
        """
        model = Workspace
        fields = ["id",
                  "title",
                  "owner"]
        read_only_fields = ('id', 'owner')

    def create(self, validated_data):
        """
        Create and return a new `workspace` instance, given the validated data.
        """
        return workspace_api.create_and_save(validated_data['title'], validated_data['user'].id)



""" REST views for the blob API
"""
from django.http import Http404
from rest_framework import status
from rest_framework.exceptions import ValidationError
from rest_framework.response import Response
from rest_framework.views import APIView

import core_main_app.components.blob.api as blob_api
from core_main_app.commons import exceptions
from core_main_app.rest.blob.serializers import BlobSerializer, DeleteBlobsSerializer
from core_main_app.utils.access_control.exceptions import AccessControlError
from core_main_app.utils.file import get_file_http_response


class BlobList(APIView):
    """ List all user blobs, or create a new one.
    """

    def get(self, request):
        """ Get all user blobs

        /rest/blob/
        /rest/blob/?filename=<filename>

        Query Params:
            filename: filename

        Args:
            request:

        Returns:

        """
        try:
            # FIXME: remove?
            if request.user.is_superuser:
                blob_list = blob_api.get_all()
            else:
                blob_list = blob_api.get_all_by_user_id(user_id=request.user.id)

            # Apply filters
            filename = self.request.query_params.get('filename', None)
            if filename is not None:
                blob_list = blob_list.filter(filename=filename)

            # Serialize object
            serializer = BlobSerializer(blob_list, many=True, context={'request': request})

            # Return response
            return Response(serializer.data, status=status.HTTP_200_OK)
        except Exception as api_exception:
            content = {'message': api_exception.message}
            return Response(content, status=status.HTTP_500_INTERNAL_SERVER_ERROR)

    def post(self, request):
        """ Create blob

        Data:
            {
            "blob": "<file>",
            "filename": "<filename>"
            }

        Args:
            request:

        Returns:

        """
        try:
            # Build serializer
            serializer = BlobSerializer(data=request.data, context={'request': request})

            # Validate data
            serializer.is_valid(True)

            # Save data
            serializer.save(user=request.user)

            # Return the serialized data
            return Response(serializer.data, status=status.HTTP_201_CREATED)
        except ValidationError as validation_exception:
            content = {'message': validation_exception.detail}
            return Response(content, status=status.HTTP_400_BAD_REQUEST)
        except Exception as api_exception:
            content = {'message': api_exception.message}
            return Response(content, status=status.HTTP_500_INTERNAL_SERVER_ERROR)


class BlobDetail(APIView):
    """
    Retrieve, update or delete a blob.
    """

    def get_object(self, pk):
        """ Get blob from db

        Args:
            pk:

        Returns:

        """
        try:
            return blob_api.get_by_id(pk)
        except exceptions.DoesNotExist:
            raise Http404

    def get(self, request, pk):
        """ Retrieve blob

        Args:
            pk:

        Returns:

        """
        try:
            # Get object
            blob_object = self.get_object(pk)

            # Serialize object
            serializer = BlobSerializer(blob_object, context={'request': request})

            # Return response
            return Response(serializer.data)
        except Http404:
            content = {'message': 'Blob not found.'}
            return Response(content, status=status.HTTP_404_NOT_FOUND)
        except Exception as api_exception:
            content = {'message': api_exception.message}
            return Response(content, status=status.HTTP_500_INTERNAL_SERVER_ERROR)

    def delete(self, request, pk):
        """ Delete a blob

        Args:
            request:
            pk:

        Returns:

        """
        try:
            # Get object
            blob_object = self.get_object(pk)

            # Check rights
            if request.user.is_superuser is False and str(request.user.id) != blob_object.user_id:
                return Response(status=status.HTTP_403_FORBIDDEN)

            # delete object
            blob_api.delete(blob_object)

            return Response(status=status.HTTP_204_NO_CONTENT)
        except Http404:
            content = {'message': 'Blob not found.'}
            return Response(content, status=status.HTTP_404_NOT_FOUND)
        except Exception as api_exception:
            content = {'message': api_exception.message}
            return Response(content, status=status.HTTP_500_INTERNAL_SERVER_ERROR)


class BlobDownload(APIView):
    """
    Download a blob.
    """

    def get_object(self, pk):
        """ Get blob from db

        Args:
            pk:

        Returns:

        """
        try:
            return blob_api.get_by_id(pk)
        except exceptions.DoesNotExist:
            raise Http404

    def get(self, request, pk):
        """ Retrieve blob

        Args:
            pk:

        Returns:

        """
        try:
            # Get object
            blob_object = self.get_object(pk)

            return get_file_http_response(blob_object.blob, blob_object.filename)
        except Http404:
            content = {'message': 'Blob not found.'}
            return Response(content, status=status.HTTP_404_NOT_FOUND)
        except Exception as api_exception:
            content = {'message': api_exception.message}
            return Response(content, status=status.HTTP_500_INTERNAL_SERVER_ERROR)


class BlobDeleteList(APIView):
    """ Delete list of blobs.
    """
    def patch(self, request):
        """ Delete a list of blobs.

        /rest/blobs/delete/

        Data:
        [{"id":"<blob_id>"},{"id":"<blob_id>"}]

        Args:
            request:

        Returns:

        """
        try:
            # Serialize data
            serializer = DeleteBlobsSerializer(data=request.data, many=True, context={'request': request})

            # Validate data
            serializer.is_valid(True)

            # Get list of unique ids
            blob_ids = set([blob['id'] for blob in serializer.validated_data])

            for blob_id in blob_ids:
                # Get blob with its id
                blob = blob_api.get_by_id(blob_id)
                # Delete blob
                blob_api.delete(blob)

            # Return the serialized data
            return Response(status=status.HTTP_204_NO_CONTENT)
        except ValidationError as validation_exception:
            content = {'message': validation_exception.detail}
            return Response(content, status=status.HTTP_400_BAD_REQUEST)
        except AccessControlError as access_control_error:
            content = {'message': access_control_error.message}
            return Response(content, status=status.HTTP_403_FORBIDDEN)
        except Exception as api_exception:
            content = {'message': api_exception.message}
            return Response(content, status=status.HTTP_500_INTERNAL_SERVER_ERROR)



"""Unit tests for xsl transformation rest api
"""
import unittest

from django.test.testcases import SimpleTestCase
from mock.mock import patch
from rest_framework import status

import core_main_app.rest.xsl_transformation.views as xsl_views
from core_main_app.components.xsl_transformation.models import XslTransformation
from core_main_app.utils.tests_tools.MockUser import create_mock_user
from core_main_app.utils.tests_tools.RequestMock import RequestMock


class TestGetAllXslTransformation(SimpleTestCase):
    def setUp(self):
        super(TestGetAllXslTransformation, self).setUp()
        self.data = None

    @patch.object(XslTransformation, 'get_all')
    def test_get_all_xsl_document_returns_status_200_with_no_permission_needed(self, mock_get_all):
        # Arrange
        user = create_mock_user('0')

        # Act
        response = RequestMock.do_request_get(xsl_views.XslTransformationList.as_view(), user, self.data)

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)


class TestPostXslTransformation(SimpleTestCase):
    def setUp(self):
        super(TestPostXslTransformation, self).setUp()
        self.data = None

    @patch.object(XslTransformation, 'save_object')
    def test_post_xsl_document_returns_status_403_if_user_is_not_admin(self, mock_save):
        # Arrange
        user = create_mock_user('0')

        # Act
        response = RequestMock.do_request_post(xsl_views.XslTransformationList.as_view(), user, self.data)

        # Assert
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    @unittest.skip("Not working on Jenkins")
    @patch.object(XslTransformation, 'save_object')
    def test_post_xsl_document_returns_status_400_if_data_are_not_valid_with_admin_user(self, mock_save):
        # Arrange
        user = create_mock_user('0', True, True)

        # Act
        response = RequestMock.do_request_post(xsl_views.XslTransformationList.as_view(), user, self.data)

        # Assert
        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)

    @unittest.skip("Not working on Jenkins")
    @patch.object(XslTransformation, 'save_object')
    def test_post_xsl_document_returns_status_201_if_data_are_valid_with_admin_user(self, mock_save):
        # Arrange
        user = create_mock_user('0', True, True)
        self.data = {
            'name': 'name',
            'filename': 'filename.xsd',
            'content': '<?xml version=\"1.0\" encoding=\"UTF-8\"?>'
                       '<xsl:stylesheet xmlns:xsl=\"http://www.w3.org/1999/XSL/Transform\" version=\"1.0\">'
                       '<xsl:template></xsl:template></xsl:stylesheet>'
        }

        # Act
        response = RequestMock.do_request_post(xsl_views.XslTransformationList.as_view(), user, self.data)

        # Assert
        self.assertEqual(response.status_code, status.HTTP_201_CREATED)



"""
    The API contains the available function to access, create and edit a user
"""
from django.contrib.auth.models import User


def get_all_users():
    """ Return all Users.

    Returns:
        List of Users

    """
    return User.objects.all()


def get_user_by_username(username):
    """Return a user given its username.

        Args:
            username (str): Given username

        Returns:
            User object
    """
    return User.objects.get(username=username)


def get_user_by_id(user_id):
    """Return a user given its primary key.

        Args:
            user_id (str): Given user id

        Returns:
            User object
    """
    return User.objects.get(pk=user_id)


def create_and_save_user(username, password, first_name, last_name, email):
    """Save a user with the given parameters.

        Args:
            username (str): Given user name
            password (str): Given password
            first_name (str): Given first name
            last_name (str): Given last name
            email (str): Given email

        Returns:
            User object
    """
    user = User.objects.create_user(username=username,
                                    password=password,
                                    first_name=first_name,
                                    last_name=last_name,
                                    email=email)
    return upsert(user)


def upsert(user):
    """ Upsert user.

    Args:
        user (User): The user to be saved

    Returns:

    """
    # Has to be called separately because save from django object returns nothing
    user.save()
    return user


def get_all_users_except_list(list_user):
    """ Get all users except the given list of users.

    Args:
        list_user

    Returns:
    """
    return get_all_users_except_list_id([str(user.id) for user in list_user])


def get_all_users_except_list_id(list_user_ids):
    """ Get all users except the given list of user ids.

    Args:
        list_user_ids

    Returns:
    """
    return User.objects.exclude(id__in=list_user_ids)


def get_all_users_by_list_id(list_user_ids):
    """ Get all users by the given list of user ids.

    Args:
        list_user_ids

    Returns:
    """
    return User.objects.filter(id__in=list_user_ids)


def get_id_username_dict(list_user):
    """ Get a usable key-value list

    Args:
        list_user:

    Returns:

    """
    return dict((str(x.id), x.username) for x in list_user)



""" Fixtures files for Workspace
"""

from core_main_app.utils.integration_tests.fixture_interface import FixtureInterface
from core_main_app.components.workspace import api as workspace_api
from core_main_app.components.group import api as group_api
from core_main_app.permissions import api as permission_api


class WorkspaceFixtures(FixtureInterface):
    """ Workspace Fixture
    """
    def insert_data(self):
        pass

    @staticmethod
    def create_workspace(owner_id, title):
        return workspace_api.create_and_save(title, owner_id)

    @staticmethod
    def create_global_workspace(title):
        workspace = workspace_api.create_and_save(title, is_public=True)
        permission_api.add_permission_to_group(group_api.get_anonymous_group(), workspace.read_perm_id)
        permission_api.add_permission_to_group(group_api.get_default_group(), workspace.read_perm_id)
        return workspace



"""Utils for permissions
"""


def get_formatted_name(permission):
    """Return the formatted name from a permission.

    Args:
        permission:

    Returns:

    """
    return "Can {}".format(permission.replace("_", " "))




""" Data model
"""

from django_mongoengine import fields
from mongoengine import errors as mongoengine_errors
from mongoengine.queryset.base import NULLIFY

from core_main_app.commons import exceptions
from core_main_app.components.abstract_data.models import AbstractData
from core_main_app.components.template.models import Template
from core_main_app.components.workspace.models import Workspace


# TODO: Create publication workflow manager
# TODO: execute_query / execute_query_full_result -> use find method (RETURN FULL OBJECT)


class Data(AbstractData):
    """ Data object
    """
    template = fields.ReferenceField(Template, blank=False)
    user_id = fields.StringField()
    workspace = fields.ReferenceField(Workspace, reverse_delete_rule=NULLIFY, blank=True)

    @staticmethod
    def get_all(order_by_field=None):
        """ Get all data.

        Args:
            order_by_field: Order by field.

        Returns:

        """
        return Data.objects.order_by(order_by_field)

    @staticmethod
    def get_all_by_user_id(user_id, order_by_field=None):
        """ Get all data relative to the given user id

        Args:
            user_id:
            order_by_field: Order by field.

        Returns:

        """
        return Data.objects(user_id=str(user_id)).order_by(order_by_field)

    @staticmethod
    def get_all_except_user_id(user_id):
        """ Get all data non relative to the given user id

        Args:
            user_id:

        Returns:

        """
        return Data.objects(user_id__nin=str(user_id)).all()

    @staticmethod
    def get_all_by_id_list(list_id):
        """ Return the object with the given list id.

        Args:
            list_id:

        Returns:
            Object collection
        """
        return Data.objects(pk__in=list_id)

    @staticmethod
    def get_by_id(data_id):
        """ Return the object with the given id.

        Args:
            data_id:

        Returns:
            Data (obj): Data object with the given id

        """
        try:
            return Data.objects.get(pk=str(data_id))
        except mongoengine_errors.DoesNotExist as e:
            raise exceptions.DoesNotExist(e.message)
        except Exception as ex:
            raise exceptions.ModelError(ex.message)

    @staticmethod
    def execute_query(query, order_by_field=None):
        """Execute a query.

        Args:
            query:
            order_by_field: Order by field.

        Returns:

        """
        return Data.objects(__raw__=query).order_by(order_by_field)

    @staticmethod
    def get_all_by_workspace(workspace):
        """ Get all data that belong to the workspace.

        Args:
            workspace:

        Returns:

        """
        return Data.objects(workspace=workspace).all()

    @staticmethod
    def get_all_by_list_workspace(list_workspace):
        """ Get all data that belong to the list of workspace.

        Args:
            list_workspace:

        Returns:

        """
        return Data.objects(workspace__in=list_workspace).all()

    @staticmethod
    def get_all_by_list_template(list_template):
        """ Get all data that belong to the list of template.

        Args:
            list_template:

        Returns:

        """
        return Data.objects(template__in=list_template).all()

    @staticmethod
    def aggregate(pipeline):
        """Execute an aggregate on the Data collection.

        Args:
            pipeline:

        Returns:

        """
        return Data.objects.aggregate(*pipeline)



""" Serializers for Django Group
"""
from rest_framework import serializers
from django.contrib.auth.models import Group


class GroupSerializer(serializers.ModelSerializer):
    """ Group serializer
    """
    class Meta:
        """ Meta
        """
        model = Group
        fields = ('id', 'name')


"""
Template API
"""
from core_main_app.commons import exceptions
from core_main_app.components.template.models import Template
from core_main_app.utils.xml import is_schema_valid, get_hash, \
    get_template_with_server_dependencies, get_local_dependencies


def upsert(template):
    """Save or Updates the template.

    Args:
        template:

    Returns:

    """
    # Check if schema is valid
    is_schema_valid(template.content)
    # Get hash for the template
    template.hash = get_hash(template.content)
    # Register local dependencies
    _register_local_dependencies(template)
    # Save template
    return template.save()


def init_template_with_dependencies(template, dependencies_dict):
    """Initialize template content and dependencies from a dictionary.

    Args:
        template:
        dependencies_dict:

    Returns:

    """
    if dependencies_dict is not None:
        # update template content
        template.content = get_template_with_server_dependencies(template.content, dependencies_dict)

    return template


def set_display_name(template, display_name):
    """Set template display name.

    Args:
        template:
        display_name:

    Returns:

    """
    # Set display name
    template.display_name = display_name
    # Save
    template.save()


def get(template_id):
    """Get a template.

    Args:
        template_id:

    Returns:

    """
    return Template.get_by_id(template_id)


def get_all_by_hash(template_hash):
    """ Return all template having the given hash.

    Args:
        template_hash: Template hash.

    Returns:
        List of Template instance.

    """
    return Template.get_all_by_hash(template_hash)


def get_all_by_hash_list(template_hash_list):
    """ Return all template having the given hash list.

    Args:
        template_hash_list: Template hash list.

    Returns:
        List of Template instance.

    """
    return Template.get_all_by_hash_list(template_hash_list)


def get_all_by_id_list(template_id_list):
    """ Returns all template with id in list

    Args:
        template_id_list:

    Returns:

    """
    return Template.get_all_by_id_list(template_id_list)


def get_all(is_cls=True):
    """List all templates.

    Returns:

    """
    return Template.get_all(is_cls)


def delete(template):
    """Delete the template.

    Returns:

    """
    template.delete()


def _register_local_dependencies(template):
    """ Register local dependencies for the given template.

    Args:
        template: Template instance.

    Returns:

    """
    # Clean all dependencies. Template content could have been changed.
    del template.dependencies
    # Get local dependencies
    local_dependencies = get_local_dependencies(template.content)
    for local_dependency in local_dependencies:
        try:
            # get the dependency
            dependency_object = get(local_dependency)
            # add the dependency
            template.dependencies.append(dependency_object)
        except exceptions.DoesNotExist:
            pass


def get_all_templates_by_dependencies(dependency_id_list):
    """List all templates having the given dependencies.

    Args:
        dependency_id_list: List of dependency ids.

    Returns:
        List of templates.

    """

    return Template.get_all_templates_by_dependencies(dependency_id_list)



"""
Workspace model
"""
from django_mongoengine import fields, Document
from mongoengine import errors as mongoengine_errors
from mongoengine.queryset.visitor import Q

from core_main_app.commons import exceptions
from core_main_app.commons.regex import NOT_EMPTY_OR_WHITESPACES


class Workspace(Document):
    """
        Workspace class.
    """

    title = fields.StringField(blank=False, regex=NOT_EMPTY_OR_WHITESPACES)
    owner = fields.StringField(blank=True)
    read_perm_id = fields.StringField(blank=False)
    write_perm_id = fields.StringField(blank=False)
    is_public = fields.BooleanField(default=False)

    def clean(self):
        """

        Returns:
        """
        # Check the title
        if self.owner is not None and self.title.lower() == "global public workspace":
            raise exceptions.ModelError("You can't create a workspace with the title: " + self.title)

    @staticmethod
    def get_all():
        """ Get all workspaces.

        Returns:

        """
        return Workspace.objects.all()

    @staticmethod
    def get_all_by_owner(user_id):
        """ Get all workspaces created by the given user id.

        Args:
            user_id

        Returns:

        """
        return Workspace.objects(owner=str(user_id)).all()

    @staticmethod
    def get_by_id(workspace_id):
        """ Return the workspace with the given id.

        Args:
            workspace_id

        Returns:
            Workspace (obj): Workspace object with the given id

        """
        try:
            return Workspace.objects.get(pk=str(workspace_id))
        except mongoengine_errors.DoesNotExist as e:
            raise exceptions.DoesNotExist(e.message)
        except Exception as ex:
            raise exceptions.ModelError(ex.message)

    @staticmethod
    def get_all_workspaces_with_read_access_by_user_id(user_id, read_permissions):
        """ Get all workspaces with read access for the given user id.

        Args:
            user_id
            read_permissions

        Returns:

        """
        return Workspace.objects(Q(owner=str(user_id)) | Q(read_perm_id__in=read_permissions) | Q(is_public=True)).all()

    @staticmethod
    def get_all_workspaces_with_write_access_by_user_id(user_id, write_permissions):
        """ Get all workspaces with write access for the given user id.

        Args:
            user_id
            write_permissions

        Returns:

        """
        return Workspace.objects(Q(owner=str(user_id)) | Q(write_perm_id__in=write_permissions)).all()

    @staticmethod
    def get_all_workspaces_with_read_access_not_owned_by_user_id(user_id, read_permissions):
        """ Get all workspaces with read access not owned by the given user id.

        Args:
            user_id
            read_permissions

        Returns:

        """

        return Workspace.objects(Q(read_perm_id__in=read_permissions) | Q(is_public=True), owner__ne=str(user_id)).all()

    @staticmethod
    def get_all_workspaces_with_write_access_not_owned_by_user_id(user_id, write_permissions):
        """ Get all workspaces with write access not owned by the given user id.

        Args:
            user_id
            write_permissions

        Returns:

        """
        return Workspace.objects(owner__ne=str(user_id), write_perm_id__in=write_permissions).all()

    @staticmethod
    def get_all_public_workspaces():
        """ Get all public workspaces.

        Args:

        Returns:

        """
        return Workspace.objects(is_public=True).all()

    @staticmethod
    def get_all_other_public_workspaces(user_id):
        """ Get all other public workspaces.

        Args:
            user_id

        Returns:

        """
        return Workspace.objects(owner__ne=str(user_id), is_public=True).all()

    @staticmethod
    def get_non_public_workspace_owned_by_user_id(user_id):
        """ Get the non public workspaces owned by the given user id.

        Args:
            user_id

        Returns:

        """
        return Workspace.objects(owner=str(user_id), is_public=False).all()

    @staticmethod
    def get_public_workspaces_owned_by_user_id(user_id):
        """ Get the public workspaces owned the given user id.

        Args:
            user_id

        Returns:

        """
        return Workspace.objects(owner=str(user_id), is_public=True).all()

    @staticmethod
    def get_global_workspace():
        """ Get global workspace.

        Returns:
        """
        try:
            return Workspace.objects.get(owner=None, is_public=True)
        except mongoengine_errors.DoesNotExist as e:
            raise exceptions.DoesNotExist(e.message)
        except Exception as ex:
            raise exceptions.ModelError(ex.message)

    @property
    def is_global(self):
        """ Get is global.

        Returns:
        """
        return self.is_public and self.owner is None



"""Results paginator util
"""
from django.core.paginator import Paginator, PageNotAnInteger, EmptyPage
from core_main_app.settings import RESULTS_PER_PAGE


class ResultsPaginator(object):

    @staticmethod
    def get_results(results_list, page, results_per_page=RESULTS_PER_PAGE):
        # Pagination
        paginator = Paginator(results_list, results_per_page)

        try:
            results = paginator.page(int(page))
        except PageNotAnInteger:
            results = paginator.page(1)
        except EmptyPage:
            results = paginator.page(paginator.num_pages)

        return results



""" Integration Test for Blob Rest API
"""
import unittest
from os.path import join, dirname, abspath

from django.test import override_settings
from rest_framework import status

from core_main_app.components.blob import api as blob_api
from core_main_app.rest.blob import views
from core_main_app.utils.integration_tests.integration_base_test_case import MongoIntegrationBaseTestCase
from core_main_app.utils.tests_tools.MockUser import create_mock_user
from core_main_app.utils.tests_tools.RequestMock import RequestMock
from tests.components.blob.fixtures.fixtures import BlobFixtures

RESOURCES_PATH = join(dirname(abspath(__file__)), 'data')
fixture_blob = BlobFixtures()


class TestBlobList(MongoIntegrationBaseTestCase):
    fixture = fixture_blob

    def setUp(self):
        super(TestBlobList, self).setUp()
        self.data = {'filename': 'file.txt',
                     'blob': open(join(RESOURCES_PATH, 'test.txt'), 'r')}

    @override_settings(ROOT_URLCONF="core_main_app.urls")
    def test_get_returns_http_200(self):
        # Arrange
        user = create_mock_user('1')

        # Act
        response = RequestMock.do_request_get(views.BlobList.as_view(),
                                              user)

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)

    @override_settings(ROOT_URLCONF="core_main_app.urls")
    def test_get_returns_all_user_blobs(self):
        # Arrange
        user = create_mock_user('1')

        # Act
        response = RequestMock.do_request_get(views.BlobList.as_view(),
                                              user)

        # Assert
        self.assertEqual(len(response.data), 2)

    @override_settings(ROOT_URLCONF="core_main_app.urls")
    def test_get_as_superuser_returns_all_user_blobs(self):
        # Arrange
        user = create_mock_user('1', is_superuser=True)

        # Act
        response = RequestMock.do_request_get(views.BlobList.as_view(),
                                              user)

        # Assert
        self.assertEqual(len(response.data), 3)

    @override_settings(ROOT_URLCONF="core_main_app.urls")
    def test_get_filtered_by_correct_name_returns_http_200(self):
        # Arrange
        user = create_mock_user('1', is_superuser=True)

        # Act
        response = RequestMock.do_request_get(views.BlobList.as_view(),
                                              user,
                                              data={'filename': self.fixture.blob_1.filename})

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)

    @override_settings(ROOT_URLCONF="core_main_app.urls")
    def test_get_filtered_by_correct_name_returns_correct_blob(self):
        # Arrange
        user = create_mock_user('1', is_superuser=True)

        # Act
        response = RequestMock.do_request_get(views.BlobList.as_view(),
                                              user,
                                              data={'filename': self.fixture.blob_1.filename})

        # Assert
        self.assertEqual(response.data[0]['filename'], self.fixture.blob_1.filename)

    @override_settings(ROOT_URLCONF="core_main_app.urls")
    def test_get_filtered_by_incorrect_name_returns_http_200(self):
        # Arrange
        user = create_mock_user('1', is_superuser=True)

        # Act
        response = RequestMock.do_request_get(views.BlobList.as_view(),
                                              user,
                                              data={'filename': 'incorrect'})

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)

    @override_settings(ROOT_URLCONF="core_main_app.urls")
    def test_get_filtered_by_incorrect_name_returns_empty_list(self):
        # Arrange
        user = create_mock_user('1', is_superuser=True)

        # Act
        response = RequestMock.do_request_get(views.BlobList.as_view(),
                                              user,
                                              data={'filename': 'incorrect'})

        # Assert
        self.assertEqual(len(response.data), 0)

    @unittest.skip("need to mock GridFS Blob Host")
    @override_settings(ROOT_URLCONF="core_main_app.urls")
    def test_post_returns_http_201(self):
        # Arrange
        user = create_mock_user('1')

        # Act
        response = RequestMock.do_request_post(views.BlobList.as_view(),
                                               user,
                                               data=self.data)

        # Assert
        self.assertEqual(response.status_code, status.HTTP_201_CREATED)

    @unittest.skip("need to mock GridFS Blob Host")
    @override_settings(ROOT_URLCONF="core_main_app.urls")
    def test_post_adds_an_entry_in_database(self):
        # Arrange
        user = create_mock_user('1')

        # Act
        response = RequestMock.do_request_post(views.BlobList.as_view(),
                                               user,
                                               data=self.data)

        # Assert
        self.assertEqual(len(blob_api.get_all()), 4)

    @override_settings(ROOT_URLCONF="core_main_app.urls")
    def test_post_incorrect_file_parameter_returns_http_400(self):
        # Arrange
        user = create_mock_user('1')
        self.data['blob'] = 'test.txt'

        # Act
        response = RequestMock.do_request_post(views.BlobList.as_view(),
                                               user,
                                               data=self.data)

        # Assert
        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)


class TestBlobDetail(MongoIntegrationBaseTestCase):
    fixture = fixture_blob

    def setUp(self):
        super(TestBlobDetail, self).setUp()

    @override_settings(ROOT_URLCONF="core_main_app.urls")
    def test_get_returns_http_200(self):
        # Arrange
        user = create_mock_user('1')

        # Act
        response = RequestMock.do_request_get(views.BlobDetail.as_view(),
                                              user,
                                              param={'pk': str(self.fixture.blob_1.id)})

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)

    @override_settings(ROOT_URLCONF="core_main_app.urls")
    def test_get_returns_blob(self):
        # Arrange
        user = create_mock_user('1')

        # Act
        response = RequestMock.do_request_get(views.BlobDetail.as_view(),
                                              user,
                                              param={'pk': str(self.fixture.blob_1.id)})

        # Assert
        self.assertEqual(response.data['filename'], self.fixture.blob_1.filename)

    @override_settings(ROOT_URLCONF="core_main_app.urls")
    def test_get_wrong_id_returns_http_404(self):
        # Arrange
        user = create_mock_user('1')

        # Act
        response = RequestMock.do_request_get(views.BlobDetail.as_view(),
                                              user,
                                              param={'pk': '507f1f77bcf86cd799439011'})

        # Assert
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

    @override_settings(ROOT_URLCONF="core_main_app.urls")
    def test_get_other_user_blob_returns_http_200(self):
        # Arrange
        user = create_mock_user('2')

        # Act
        response = RequestMock.do_request_get(views.BlobDetail.as_view(),
                                              user,
                                              param={'pk': str(self.fixture.blob_1.id)})

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)

    @unittest.skip("need to mock GridFS Blob Host")
    @override_settings(ROOT_URLCONF="core_main_app.urls")
    def test_delete_returns_http_204(self):
        # Arrange
        user = create_mock_user('1')

        # Act
        response = RequestMock.do_request_delete(views.BlobDetail.as_view(),
                                                 user,
                                                 param={'pk': str(self.fixture.blob_1.id)})

        # Assert
        self.assertEqual(response.status_code, status.HTTP_204_NO_CONTENT)

    @unittest.skip("need to mock GridFS Blob Host")
    @override_settings(ROOT_URLCONF="core_main_app.urls")
    def test_delete_deletes_one_blob_from_database(self):
        # Arrange
        user = create_mock_user('1')

        # Act
        response = RequestMock.do_request_delete(views.BlobDetail.as_view(),
                                                 user,
                                                 param={'pk': str(self.fixture.blob_1.id)})

        # Assert
        self.assertEqual(len(blob_api.get_all()), 2)

    @override_settings(ROOT_URLCONF="core_main_app.urls")
    def test_delete_wrong_id_returns_http_404(self):
        # Arrange
        user = create_mock_user('1')

        # Act
        response = RequestMock.do_request_delete(views.BlobDetail.as_view(),
                                                 user,
                                                 param={'pk': '507f1f77bcf86cd799439011'})

        # Assert
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

    @override_settings(ROOT_URLCONF="core_main_app.urls")
    def test_delete_other_user_blob_returns_http_403(self):
        # Arrange
        user = create_mock_user('2')

        # Act
        response = RequestMock.do_request_delete(views.BlobDetail.as_view(),
                                                 user,
                                                 param={'pk': str(self.fixture.blob_1.id)})

        # Assert
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    @unittest.skip("need to mock GridFS Blob Host")
    @override_settings(ROOT_URLCONF="core_main_app.urls")
    def test_delete_other_user_as_superuser_returns_http_204(self):
        # Arrange
        user = create_mock_user('2', is_superuser=True)

        # Act
        response = RequestMock.do_request_delete(views.BlobDetail.as_view(),
                                                 user,
                                                 param={'pk': str(self.fixture.blob_1.id)})

        # Assert
        self.assertEqual(response.status_code, status.HTTP_204_NO_CONTENT)


class TestBlobDownload(MongoIntegrationBaseTestCase):
    fixture = fixture_blob

    def setUp(self):
        super(TestBlobDownload, self).setUp()
        self.blob = open(join(RESOURCES_PATH, 'test.txt'), 'r').read()

    @unittest.skip("need to mock GridFS Blob Host")
    @override_settings(ROOT_URLCONF="core_main_app.urls")
    def test_get_returns_http_200(self):
        # Arrange
        user = create_mock_user('1')

        # Act
        response = RequestMock.do_request_get(views.BlobDownload.as_view(),
                                              user,
                                              param={'pk': str(self.fixture.blob_1.id)})

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)

    @override_settings(ROOT_URLCONF="core_main_app.urls")
    def test_get_wrong_id_returns_http_404(self):
        # Arrange
        user = create_mock_user('1')

        # Act
        response = RequestMock.do_request_get(views.BlobDownload.as_view(),
                                              user,
                                              param={'pk': '507f1f77bcf86cd799439011'})

        # Assert
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

    @unittest.skip("need to mock GridFS Blob Host")
    @override_settings(ROOT_URLCONF="core_main_app.urls")
    def test_get_other_user_blob_returns_http_200(self):
        # Arrange
        user = create_mock_user('2')

        # Act
        response = RequestMock.do_request_get(views.BlobDownload.as_view(),
                                              user,
                                              param={'pk': str(self.fixture.blob_1.id)})

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)


class TestBlobDeleteList(MongoIntegrationBaseTestCase):
    fixture = fixture_blob

    def setUp(self):
        super(TestBlobDeleteList, self).setUp()
        self.data = [{'id': str(self.fixture.blob_1.id)},
                     {'id': str(self.fixture.blob_2.id)}]

    def test_post_a_list_containing_other_user_blob_returns_http_403(self):
        # Arrange
        user = create_mock_user('2')

        # Act
        response = RequestMock.do_request_patch(views.BlobDeleteList.as_view(),
                                                user,
                                                data=self.data)

        # Assert
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    @unittest.skip("need to mock GridFS Blob Host")
    def test_post_a_list_containing_other_user_blob_as_superuser_returns_http_204(self):
        # Arrange
        user = create_mock_user('1')

        # Act
        response = RequestMock.do_request_patch(views.BlobDeleteList.as_view(),
                                                user,
                                                data=self.data)

        # Assert
        self.assertEqual(response.status_code, status.HTTP_204_NO_CONTENT)



""" XSD Flattener Database or URL class
"""
from core_main_app.utils.urls import get_template_download_pattern
from xml_utils.xsd_flattener.xsd_flattener_url import XSDFlattenerURL
from urlparse import urlparse
from core_main_app.components.template import api as template_api
from core_main_app.commons import exceptions
from django.core.urlresolvers import reverse


class XSDFlattenerDatabaseOrURL(XSDFlattenerURL):
    """ Get the content of the dependency from the database or from the URL.
    """

    def __init__(self, xml_string, download_enabled=True):
        """Initializes the flattener

        Args:
            xml_string:
            download_enabled:
        """
        XSDFlattenerURL.__init__(self, xml_string=xml_string, download_enabled=download_enabled)

    def get_dependency_content(self, uri):
        """ Get the content of the dependency from the database or from the URL. Try to get the content from the
        database first and then try to download it from the provided URI.

        Args:
            uri: Content URI.

        Returns:
            Content.

        """
        # parse url
        url = urlparse(uri)
        # get pattern to match a template download url
        pattern = get_template_download_pattern()
        # match url
        match = pattern.match(url.path)
        # if match
        if match:
            try:
                # get pk from match
                object_id = match.group('pk')
                # get template object using pk
                template = template_api.get(object_id)
                # get template content
                content = template.content
            except (exceptions.DoesNotExist, exceptions.ModelError, Exception):
                content = super(XSDFlattenerDatabaseOrURL, self).get_dependency_content(uri)
        else:
            content = super(XSDFlattenerDatabaseOrURL, self).get_dependency_content(uri)

        return content



"""Query tool unit tests
"""
from core_main_app.utils.query.mongo.prepare import _compile_regex, _add_sub_document_root
from unittest import TestCase
import copy
import re


class TestCompileRegex(TestCase):

    def test_query_with_regex_returns_query_with_compiled_regex(self):
        # set query
        query = {"dot.notation": "/regex/"}
        # compile regex
        _compile_regex(query)
        # assert
        self.assertTrue(isinstance(query["dot.notation"], re._pattern_type))

    def test_query_with_regex_on_multiple_levels_returns_query_with_compiled_regex(self):
        # set query
        query = {"$and":
                     [
                        {"$or": [{"dot.notation.1": "/regex/"}, {"dot.notation.1.#text": "/regex/"}]},
                        {"$or": [{"dot.notation.2": {"$lt": 500}}, {"dot.notation.2": {"$lt": 500}}]}
                     ]
                 }
        # compile regex
        _compile_regex(query)
        # assert
        self.assertTrue(isinstance(query["$and"][0]["$or"][0]["dot.notation.1"], re._pattern_type))
        self.assertTrue(isinstance(query["$and"][0]["$or"][1]["dot.notation.1.#text"], re._pattern_type))
        self.assertFalse(isinstance(query["$and"][1]["$or"][0]["dot.notation.2"], re._pattern_type))
        self.assertFalse(isinstance(query["$and"][1]["$or"][1]["dot.notation.2"], re._pattern_type))

    def test_query_without_regex_returns_same_query(self):
        # set query
        query = {"dot.notation": {"$gt": 0}}
        # save original query
        query_origin = copy.deepcopy(query)
        # compile regex
        _compile_regex(query)
        # assert
        self.assertEquals(query, query_origin)

    def test_query_without_regex_on_multiple_levels_returns_same_query(self):
        # set query
        query = {"$or": [{"dot.notation": {"$gt": 0}}, {"dot.notation.#text": {"$gt": 0}}]}
        # save original query
        query_origin = copy.deepcopy(query)
        # compile regex
        _compile_regex(query)
        # assert
        self.assertEquals(query, query_origin)


class TestAddSubDocumentRoot(TestCase):

    def test_add_sub_document_root_to_query(self):
        # set query
        query = {"dot.notation": "test"}
        # set expected query
        expected_query = {"root.dot.notation": "test"}
        # compile regex
        _add_sub_document_root(query, "root")
        # assert
        self.assertEquals(query, expected_query)

    def test_add_sub_document_root_to_and_query(self):
        # set query
        query = {"$and": [{"dot.notation": {"$gt": 0}}, {"dot.notation.#text": {"$gt": 0}}]}
        # set expected query
        expected_query = {"$and": [{"root.dot.notation": {"$gt": 0}}, {"root.dot.notation.#text": {"$gt": 0}}]}
        # compile regex
        _add_sub_document_root(query, "root")
        # assert
        self.assertEquals(query, expected_query)

    def test_add_sub_document_root_to_or_query(self):
        # set query
        query = {"$or": [{"dot.notation": {"$gt": 0}}, {"dot.notation.#text": {"$gt": 0}}]}
        # set expected query
        expected_query = {"$or": [{"root.dot.notation": {"$gt": 0}}, {"root.dot.notation.#text": {"$gt": 0}}]}
        # compile regex
        _add_sub_document_root(query, "root")
        # assert
        self.assertEquals(query, expected_query)

    def test_add_sub_document_root_to_and_or_query(self):
        # set query
        query = {"$and":
                     [
                        {"$or": [{"dot.notation.1": "test"}, {"dot.notation.1.#text": "/regex/"}]},
                        {"$or": [{"dot.notation.2": {"$lt": 500}}, {"dot.notation.2": {"$lt": 500}}]}
                     ]
                 }
        # set expected query
        expected_query = {"$and":
                             [
                                {"$or": [{"root.dot.notation.1": "test"}, {"root.dot.notation.1.#text": "/regex/"}]},
                                {"$or": [{"root.dot.notation.2": {"$lt": 500}}, {"root.dot.notation.2": {"$lt": 500}}]}
                             ]
                         }
        # compile regex
        _add_sub_document_root(query, "root")
        # assert
        self.assertEquals(query, expected_query)



"""Mock user for tests
"""
from mock.mock import Mock
from django.contrib.auth.models import User


def create_mock_user(user_id, is_staff=False, is_superuser=False, has_perm=False,
                     is_anonymous=False):
    """ Mock a User.

        Args:
            user_id:
            is_staff:
            is_superuser:
            has_perm:
            is_anonymous:

    """
    mock_user = Mock(spec=User)
    mock_user.id = user_id
    mock_user.is_staff = is_staff
    mock_user.is_superuser = is_superuser
    mock_user.has_perm.return_value = has_perm
    mock_user.is_anonymous = is_anonymous

    return mock_user



""" Core main app settings
"""
import os
from os.path import join

from django.conf import settings

if not settings.configured:
    settings.configure()

SECRET_KEY = getattr(settings, 'SECRET_KEY', '<secret_key>')
""" str: Django application secret key
"""

SERVER_URI = getattr(settings, 'SERVER_URI', "http://127.0.0.1:8000")
""" str: Server URI for import reference
"""

INSTALLED_APPS = getattr(settings, 'INSTALLED_APPS', [])
""" list: List of apps installed
"""

# Website configuration
WEBSITE_COMPACT_TITLE = getattr(settings, 'WEBSITE_COMPACT_TITLE', "Project")
""" str: Website title
"""

# Choose from:  black, black-light, blue, blue-light, green, green-light, purple, purple-light, red, red-light, yellow,
#               yellow-light
WEBSITE_ADMIN_COLOR = getattr(settings, 'WEBSITE_ADMIN_COLOR', "yellow")
""" str: color of the admin dashboard
"""

# XML
XERCES_VALIDATION = getattr(settings, 'XERCES_VALIDATION', False)
""":py:class:`bool`: Enables Xerces validation (requires additional packages).
"""

# GridFS
GRIDFS_DATA_COLLECTION = getattr(settings, 'GRIDFS_DATA_COLLECTION', 'fs_data')
""" str: Collection name for file storage in MongoDB.
"""

# Celery configuration
USE_BACKGROUND_TASK = getattr(settings, 'USE_BACKGROUND_TASK', False)
""" bool: Define use of celery for background tasks
"""

BROKER_URL = getattr(settings, 'BROKER_URL', 'redis://localhost:6379/0')
""" str: Celery broker address
"""

broker_transport_default = {
    "visibility_timeout": 3600,
    'fanout_prefix': True,
    'fanout_patterns': True
}
BROKER_TRANSPORT_OPTIONS = getattr(settings, 'BROKER_TRANSPORT_OPTIONS', broker_transport_default)
""" dict: Celery broker options
"""

CELERY_RESULT_BACKEND = getattr(settings, 'CELERY_RESULT_BACKEND', 'redis://localhost:6379/0')
""" str: Celery backend for results
"""

# SMTP Configuration
USE_EMAIL = getattr(settings, 'USE_EMAIL', False)
""" bool: Activate email sending on the platform
"""

SERVER_EMAIL = getattr(settings, 'SERVER_EMAIL', 'noreply@curator.org')
""" str: Email address sending the message.
"""

ADMINS = getattr(settings, 'ADMINS', [('admin', 'admin@curator.org')])
""" list: Email addresses of admins.
"""

MANAGERS = getattr(settings, 'MANAGERS', [('manager', 'moderator@curator.org')])
""" list: Email addresses of moderators (managers). 
"""

EMAIL_SUBJECT_PREFIX = getattr(settings, 'EMAIL_SUBJECT_PREFIX', "[CURATOR] ")
""" str: Prefix for easy classification of emails.
"""

# Replace by your own values
MONGO_USER = getattr(settings, 'MONGO_USER', "mgi_user")
MONGO_PASSWORD = getattr(settings, 'MONGO_PASSWORD', "mgi_password")
DB_NAME = getattr(settings, 'DB_NAME', "mgi")

mongodb_uri_default = "mongodb://" + MONGO_USER + ":" + MONGO_PASSWORD + "@localhost/" + DB_NAME
MONGODB_URI = getattr(settings, 'MONGODB_URI', mongodb_uri_default)

USE_TZ = getattr(settings, 'USE_TZ', True)
""" bool: Activate use of timezones
"""

SEARCHABLE_DATA_OCCURRENCES_LIMIT = getattr(settings, 'SEARCHABLE_DATA_OCCURRENCES_LIMIT', None)
""" :py:class:`int` | :py:attr:`None`: Limit for number of occurent to be returned by a search
"""

BLOB_HOST = getattr(settings, 'BLOB_HOST', 'GRIDFS')
""" str: Type of file storage
"""

BLOB_HOST_URI = getattr(settings, 'BLOB_HOST_URI', MONGODB_URI)
""" str: File storage system URI
"""

BLOB_HOST_USER = getattr(settings, 'BLOB_HOST_USER', None)
""" str: User for file storage
"""

BLOB_HOST_PASSWORD = getattr(settings, 'BLOB_HOST_PASSWORD', None)
""" str: Password for file storage
"""

PASSWORD_MIN_LENGTH = getattr(settings, 'PASSWORD_MIN_LENGTH', 0)
""" int: Required minimum length of a password
"""

PASSWORD_MIN_LOWERCASE_LETTERS = getattr(settings, 'PASSWORD_MIN_LOWERCASE_LETTERS', 0)
""" int: Required number of lowercase chars in a password
"""

PASSWORD_MIN_UPPERCASE_LETTERS = getattr(settings, 'PASSWORD_MIN_UPPERCASE_LETTERS', 0)
""" int: Required number of uppercase chars in a password
"""

# Lock
LOCK_OBJECT_TTL = getattr(settings, 'LOCK_OBJECT_TTL', 600)  # 10 min
""" int: Lock duration on files
"""

# Results per page for paginator
RESULTS_PER_PAGE = getattr(settings, 'RESULTS_PER_PAGE', 10)
""" int: Results per page
"""

CAN_SET_PUBLIC_DATA_TO_PRIVATE = getattr(settings, 'CAN_SET_PUBLIC_DATA_TO_PRIVATE', True)
""" bool: Can set public data to private
"""

DEFAULT_DATA_RENDERING_XSLT = getattr(settings, 'DEFAULT_DATA_RENDERING_XSLT',
                                      join('core_main_app', 'common', 'xsl', 'xml2html.xsl'))
""" str: Path to default XSLT to render data
"""

# Can anonymous access public data
CAN_ANONYMOUS_ACCESS_PUBLIC_DATA = getattr(settings, 'CAN_ANONYMOUS_ACCESS_PUBLIC_DATA', False)
""" boolean: Can anonymous user access public data
"""

DISPLAY_NIST_HEADERS = getattr(settings, 'DISPLAY_NIST_HEADERS', False)
""" boolean: HTML pages show the NIST headers/footers
"""

# Locale folder
BASE_DIR = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
LOCALE_PATHS = (
    os.path.join(BASE_DIR, 'core_main_app/locale'),
)



"""Admin AJAX views
"""
import HTMLParser
import json

from django.core.urlresolvers import reverse_lazy
from django.http.response import HttpResponse, HttpResponseBadRequest

from core_main_app.commons.exceptions import NotUniqueError
from core_main_app.components.template.api import init_template_with_dependencies
from core_main_app.components.template.models import Template
from core_main_app.components.template_version_manager import api as template_version_manager_api
from core_main_app.components.template_version_manager.models import TemplateVersionManager
from core_main_app.components.version_manager import api as version_manager_api
from core_main_app.components.xsl_transformation import api as xsl_transformation_api
from core_main_app.components.xsl_transformation.models import XslTransformation
from core_main_app.views.admin.forms import EditXSLTForm
from core_main_app.views.common.ajax import EditObjectModalView, DeleteObjectModalView


def resolve_dependencies(request):
    """Resolve import/includes to avoid local references.

    Args:
        request:

    Returns:

    """
    try:
        # Get the parameters
        name = request.POST.get('name', None)
        version_manager_id = request.POST.get('version_manager_id', '')
        filename = request.POST['filename']
        xsd_content = request.POST['xsd_content']
        schema_locations = request.POST.getlist('schemaLocations[]')
        dependencies = request.POST.getlist('dependencies[]')

        # create new object
        template = Template(filename=filename, content=_get_xsd_content_from_html(xsd_content))
        init_template_with_dependencies(template, _get_dependencies_dict(schema_locations, dependencies))

        # get the version manager or create a new one
        if version_manager_id != '':
            template_version_manager = version_manager_api.get(version_manager_id)
        else:
            template_version_manager = TemplateVersionManager(title=name)
        template_version_manager_api.insert(template_version_manager, template)
    except Exception, e:
        return HttpResponseBadRequest(e.message)

    return HttpResponse(json.dumps({}), content_type='application/javascript')


def _get_dependencies_dict(schema_locations, dependencies):
    """Build a dict from lists of schema locations and dependencies.

    Args:
        schema_locations:
        dependencies:

    Returns:

    """
    string_to_python_dependencies = []
    # transform 'None' into python None
    for dependency in dependencies:
        if dependency == 'None':
            string_to_python_dependencies.append(None)
        else:
            string_to_python_dependencies.append(dependency)
    return dict(zip(schema_locations, string_to_python_dependencies))


def _get_xsd_content_from_html(xsd_content):
    """Decode XSD content from HTML.

    Args:
        xsd_content:

    Returns:

    """
    html_parser = HTMLParser.HTMLParser()
    xsd_content = str(html_parser.unescape(xsd_content).encode("utf-8"))
    return xsd_content


class EditXSLTView(EditObjectModalView):
    form_class = EditXSLTForm
    model = XslTransformation
    success_url = reverse_lazy("admin:core_main_app_xslt")
    success_message = 'XSLT edited with success.'

    def _save(self, form):
        # Save treatment.
        try:
            xsl_transformation_api.upsert(self.object)
        except NotUniqueError:
            form.add_error(None, "An object with the same name already exists. Please choose "
                                 "another name.")
        except Exception, e:
            form.add_error(None, e.message)


class DeleteXSLTView(DeleteObjectModalView):
    model = XslTransformation
    success_url = reverse_lazy("admin:core_main_app_xslt")
    success_message = 'XSLT deleted with success.'
    field_for_name = 'name'

    def _delete(self, request, *args, **kwargs):
        # Delete treatment.
        xsl_transformation_api.delete(self.object)



""" BLOB API
"""
import core_main_app.commons.exceptions as exceptions
from core_main_app.components.blob.models import Blob


def insert(blob):
    """ Insert the blob in the blob repository.

    Args:
        blob:

    Returns:

    """
    # if blob is not set
    if blob.blob is None:
        raise exceptions.ApiError("Unable to save the blob: blob field is not set.")
    # save blob on blob host
    blob.save_blob()
    # save blob in database
    return blob.save()


def delete(blob):
    """ Delete the blob.

    Args:
        blob:

    Returns:

    """
    # delete blob on blob host
    blob.delete_blob()
    # delete blob in database
    return blob.delete()


def get_by_id(blob_id):
    """ Return blob by its id.

    Args:
        blob_id:

    Returns:

    """
    return Blob.get_by_id(blob_id)


def get_all():
    """ Return all blobs.

    Args:

    Returns:
        List of Blob instances.

    """
    return Blob.get_all()


def get_all_by_user_id(user_id):
    """ Return all blobs by user.

    Args:
        user_id: User id.

    Returns:
        List of Blob instances for the given user id.

    """
    return Blob.get_all_by_user_id(user_id)


def get_all_except_user_id(user_id):
    """ Return all blobs except the ones of user.

    Args:
        user_id: User id.

    Returns:
        List of Blob instances except the given user id.

    """
    return Blob.get_all_except_user_id(user_id)



""" Url router for the administration site
"""
from django.conf.urls import url
from django.contrib import admin
from django.contrib.sites.models import Site
from django.core.urlresolvers import reverse_lazy
from django.views.generic.base import RedirectView
from core_main_app.utils.rendering import admin_render
from core_main_app.views.admin import views as admin_views, ajax as admin_ajax
from core_main_app.views.common import views as common_views

admin_urls = [
    url(r'^login', RedirectView.as_view(url=reverse_lazy("core_main_app_login"))),
    url(r'^logout', RedirectView.as_view(url=reverse_lazy("core_main_app_logout"))),

    url(r'^data$', common_views.ViewData.as_view(administration=True,
                                                template='core_main_app/admin/data/detail.html'),
        name='core_main_app_data_detail'),
    url(r'^templates$', admin_views.manage_templates, name='core_main_app_templates'),
    url(r'^template/upload$', admin_views.upload_template,
        name='core_main_app_upload_template'),
    url(r'^template/upload/(?P<version_manager_id>\w+)', admin_views.upload_template_version,
        name='core_main_app_upload_template_version'),
    url(r'^template/versions/(?P<version_manager_id>\w+)', admin_views.manage_template_versions,
        name='core_main_app_manage_template_versions'),
    url(r'^template/xslt/(?P<template_id>\w+)',
        common_views.TemplateXSLRenderingView.as_view(
            rendering=admin_render,
            template_name="core_main_app/admin/templates_xslt/main.html",
            save_redirect="admin:core_main_app_manage_template_versions",
            back_to_url="admin:core_main_app_manage_template_versions"
        ),
        name='core_main_app_template_xslt'),
    url(r'^dashboard$', admin_views.admin_home, name='core_main_app_admin_home'),

    url(r'^template/resolve-dependencies', admin_ajax.resolve_dependencies,
        name='core_main_app_resolve_dependencies'),
    url(r'^xslt/(?P<pk>[\w-]+)/edit/$', admin_ajax.EditXSLTView.as_view(),
        name='core_main_app_edit_xslt'),
    url(r'^xslt$', admin_views.XSLTView.as_view(), name='core_main_app_xslt'),
    url(r'^xslt/upload$', admin_views.UploadXSLTView.as_view(), name='core_main_app_upload_xslt'),
    url(r'^xslt/(?P<pk>[\w-]+)/delete/$',
        admin_ajax.DeleteXSLTView.as_view(),
        name='core_main_app_delete_xslt'),
    url(r'^edit-rights/(?P<workspace_id>\w+)$',
        common_views.EditWorkspaceRights.as_view(
            administration=True,
            template="core_main_app/admin/workspaces/edit_rights.html"),
        name='core_main_edit_rights_workspace'),
]

urls = admin.site.get_urls()
admin.site.get_urls = lambda: admin_urls + urls

# Admin part for the Site model is not useful in this application
admin.site.unregister(Site)



""" REST views for the workspace API
"""
from rest_framework import status
from rest_framework.decorators import api_view
from rest_framework.response import Response
from rest_framework.views import APIView
from rest_framework.exceptions import ValidationError

import core_main_app.components.workspace.api as workspace_api
import core_main_app.components.user.api as user_api
import core_main_app.components.group.api as group_api
from core_main_app.commons import exceptions
from core_main_app.rest.workspace.serializers import WorkspaceSerializer
from core_main_app.rest.user.serializers import UserSerializer
from core_main_app.rest.group.serializers import GroupSerializer
from core_main_app.utils.access_control.exceptions import AccessControlError


class WorkspaceList(APIView):
    """ List all user workspace, or create a new one.
    """

    def get(self, request):
        """ Get all user workspaces.

        Args:
            request:

        Returns:

        """
        try:
            if request.user.is_superuser:
                workspace_list = workspace_api.get_all()
            else:
                workspace_list = workspace_api.get_all_by_owner(request.user)

            # Serialize object
            serializer = WorkspaceSerializer(workspace_list, many=True)

            # Return response
            return Response(serializer.data, status=status.HTTP_200_OK)
        except Exception as api_exception:
            content = {'message': api_exception.message}
            return Response(content, status=status.HTTP_500_INTERNAL_SERVER_ERROR)

    def post(self, request):
        """ Create workspace.

        Args:
            request:

        Returns:

        """
        try:
            # Build serializer
            serializer = WorkspaceSerializer(data=request.data)

            # Validate data
            serializer.is_valid(True)

            # Save data
            serializer.save(user=request.user)

            return Response(serializer.data, status=status.HTTP_201_CREATED)
        except ValidationError as validation_exception:
            content = {'message': validation_exception.detail}
            return Response(content, status=status.HTTP_400_BAD_REQUEST)
        except exceptions.ModelError as validation_exception:
            content = {'message': validation_exception.message}
            return Response(content, status=status.HTTP_400_BAD_REQUEST)
        except exceptions.NotUniqueError as validation_exception:
            content = {'message': validation_exception.message}
            return Response(content, status=status.HTTP_400_BAD_REQUEST)
        except KeyError as validation_exception:
            content = {'message': validation_exception}
            return Response(content, status=status.HTTP_400_BAD_REQUEST)
        except Exception as api_exception:
            content = {'message': api_exception.message}
            return Response(content, status=status.HTTP_500_INTERNAL_SERVER_ERROR)


class WorkspaceDetail(APIView):
    """ Workspace Detail.
    """

    def get(self, request, pk):
        """ Retrieve workspace

        Args:
            request:
            pk:

        Returns:

        """
        try:
            # Get object
            workspace_object = workspace_api.get_by_id(pk)

            # Serialize object
            serializer = WorkspaceSerializer(workspace_object)

            # Return response
            return Response(serializer.data)
        except exceptions.DoesNotExist:
            content = {'message': 'Workspace not found.'}
            return Response(content, status=status.HTTP_404_NOT_FOUND)
        except Exception as api_exception:
            content = {'message': api_exception.message}
            return Response(content, status=status.HTTP_500_INTERNAL_SERVER_ERROR)

    def delete(self, request, pk):
        """ Delete a workspace

        Args:
            request:
            pk:

        Returns:

        """
        try:
            # Get object
            workspace_object = workspace_api.get_by_id(pk)

            # delete object
            workspace_api.delete(workspace_object, request.user)

            # Return response
            return Response(status=status.HTTP_204_NO_CONTENT)
        except exceptions.DoesNotExist:
            content = {'message': 'Workspace not found.'}
            return Response(content, status=status.HTTP_404_NOT_FOUND)
        except AccessControlError as ace:
            content = {'message': ace.message}
            return Response(content, status=status.HTTP_403_FORBIDDEN)
        except Exception as api_exception:
            content = {'message': api_exception.message}
            return Response(content, status=status.HTTP_500_INTERNAL_SERVER_ERROR)


@api_view(['GET'])
def get_workspaces_with_read_access(request):
    """ Get all workspaces with read access.

        /rest/workspace/read_access

        Args:
            request:

        Returns:

    """
    return _list_of_workspaces_to_response(workspace_api.get_all_workspaces_with_read_access_by_user(request.user))


@api_view(['GET'])
def get_workspaces_with_write_access(request):
    """ Get all workspaces with write access.

        /rest/workspace/write_access

        Args:
            request:

        Returns:

    """
    return _list_of_workspaces_to_response(workspace_api.get_all_workspaces_with_write_access_by_user(request.user))


def _list_of_workspaces_to_response(func):
    """ Serialize and generate response the list of workspaces you can get with func method.

        Args:
            func:

        Returns:
    """
    try:
        # Get object list
        workspace_object_list = func

        # Serialize object
        return_value = WorkspaceSerializer(workspace_object_list, many=True)

        # Return response
        return Response(return_value.data, status=status.HTTP_200_OK)

    except Exception as api_exception:
        content = {'message': api_exception.message}
        return Response(content, status=status.HTTP_500_INTERNAL_SERVER_ERROR)


@api_view(['GET'])
def is_workspace_public(request, pk):
    """ Is the workspace public.

        Args:
            request:
            pk:

        Returns:

    """
    try:
        # Get object
        workspace_object = workspace_api.get_by_id(pk)

        # Return response
        return Response(workspace_api.is_workspace_public(workspace_object), status=status.HTTP_200_OK)
    except exceptions.DoesNotExist:
        content = {'message': 'Workspace not found.'}
        return Response(content, status=status.HTTP_404_NOT_FOUND)
    except Exception as api_exception:
        content = {'message': api_exception.message}
        return Response(content, status=status.HTTP_500_INTERNAL_SERVER_ERROR)


@api_view(['PATCH'])
def set_workspace_public(request, pk):
    """ Set the workspace public.

        Args:
            request:
            pk:

        Returns:

    """
    try:
        # Get object
        workspace_object = workspace_api.get_by_id(pk)

        # Set the workspace public
        workspace_api.set_workspace_public(workspace_object, request.user)

        # Return response
        return Response(status=status.HTTP_200_OK)
    except exceptions.DoesNotExist:
        content = {'message': 'Workspace not found.'}
        return Response(content, status=status.HTTP_404_NOT_FOUND)
    except AccessControlError as ace:
        content = {'message': ace.message}
        return Response(content, status=status.HTTP_403_FORBIDDEN)
    except Exception as api_exception:
        content = {'message': api_exception.message}
        return Response(content, status=status.HTTP_500_INTERNAL_SERVER_ERROR)


@api_view(['GET'])
def get_list_user_can_write_workspace(request, pk):
    """ Get list of users that have write access to workspace.

    Args:
        request:
        pk:

    Returns:

    """
    return _list_of_users_or_groups_to_response(pk, workspace_api.get_list_user_can_write_workspace,
                                                request.user, UserSerializer)


@api_view(['GET'])
def get_list_user_can_read_workspace(request, pk):
    """ Get list of users that have read access to workspace.

    Args:
        request:
        pk:

    Returns:

    """
    return _list_of_users_or_groups_to_response(pk, workspace_api.get_list_user_can_read_workspace,
                                                request.user, UserSerializer)


@api_view(['GET'])
def get_list_user_can_access_workspace(request, pk):
    """ Get list of users that have read or write access to workspace.

    Args:
        request:
        pk:

    Returns:

    """
    return _list_of_users_or_groups_to_response(pk, workspace_api.get_list_user_can_access_workspace,
                                                request.user, UserSerializer)


@api_view(['GET'])
def get_list_group_can_write_workspace(request, pk):
    """ Get list of groups that have write access to workspace.

    Args:
        request:
        pk:

    Returns:

    """
    return _list_of_users_or_groups_to_response(pk, workspace_api.get_list_group_can_write_workspace,
                                                request.user, GroupSerializer)


@api_view(['GET'])
def get_list_group_can_read_workspace(request, pk):
    """ Get list of groups that have read access to workspace.

    Args:
        request:
        pk:

    Returns:

    """
    return _list_of_users_or_groups_to_response(pk, workspace_api.get_list_group_can_read_workspace,
                                                request.user, GroupSerializer)


@api_view(['GET'])
def get_list_group_can_access_workspace(request, pk):
    """ Get list of groups that have read or write access to workspace.

    Args:
        request:
        pk:

    Returns:

    """
    return _list_of_users_or_groups_to_response(pk, workspace_api.get_list_group_can_access_workspace,
                                                request.user, GroupSerializer)


def _list_of_users_or_groups_to_response(pk, func, user, serializer):
    """ List of users or groups to response.

    Args:
        pk:
        func:
        user:

    Returns:

    """
    try:
        # Get object
        workspace_object = workspace_api.get_by_id(pk)

        # Get list User
        list_user = func(workspace_object, user)

        # Serialize object
        serializer = serializer(list_user, many=True)

        # Return response
        return Response(serializer.data, status=status.HTTP_200_OK)

    except exceptions.DoesNotExist:
        content = {'message': 'Workspace not found.'}
        return Response(content, status=status.HTTP_404_NOT_FOUND)
    except AccessControlError as ace:
        content = {'message': ace.message}
        return Response(content, status=status.HTTP_403_FORBIDDEN)
    except Exception as api_exception:
        content = {'message': api_exception.message}
        return Response(content, status=status.HTTP_500_INTERNAL_SERVER_ERROR)


def _add_or_remove_to_user_or_group_right_to_workspace(request, pk, user_or_group_id, func, get_group_or_user_func):
    """ Add

    Args:
        request:
        pk:
        user_or_group_id:
        func:
        get_group_or_user_func

    Returns:
    """

    try:
        # Get object
        workspace_object = workspace_api.get_by_id(pk)

        # Get user
        user_or_group = get_group_or_user_func(user_or_group_id)

        # Add right
        func(workspace_object, user_or_group, request.user)

        # Return response
        return Response(status=status.HTTP_200_OK)

    except exceptions.DoesNotExist:
        content = {'message': 'Workspace not found.'}
        return Response(content, status=status.HTTP_404_NOT_FOUND)
    except AccessControlError as ace:
        content = {'message': ace.message}
        return Response(content, status=status.HTTP_403_FORBIDDEN)
    except Exception as api_exception:
        content = {'message': api_exception.message}
        return Response(content, status=status.HTTP_500_INTERNAL_SERVER_ERROR)


@api_view(['PATCH'])
def add_user_read_right_to_workspace(request, pk, user_id):
    """ Add to the user the read right to the workspace.

    Args:
        request:
        pk:
        user_id:

    Returns:

    """
    return _add_or_remove_to_user_or_group_right_to_workspace(request, pk, user_id,
                                                              workspace_api.add_user_read_access_to_workspace,
                                                              user_api.get_user_by_id)


@api_view(['PATCH'])
def add_user_write_right_to_workspace(request, pk, user_id):
    """ Add to the user the write right to the workspace.

    Args:
        request:
        pk:
        user_id:

    Returns:

    """
    return _add_or_remove_to_user_or_group_right_to_workspace(request, pk, user_id,
                                                              workspace_api.add_user_write_access_to_workspace,
                                                              user_api.get_user_by_id)


@api_view(['PATCH'])
def add_group_read_right_to_workspace(request, pk, group_id):
    """ Add to the group the read right to the workspace.

    Args:
        request:
        pk:
        group_id:

    Returns:

    """
    return _add_or_remove_to_user_or_group_right_to_workspace(request, pk, group_id,
                                                              workspace_api.add_group_read_access_to_workspace,
                                                              group_api.get_group_by_id)


@api_view(['PATCH'])
def add_group_write_right_to_workspace(request, pk, group_id):
    """ Add to the group the write right to the workspace.

    Args:
        request:
        pk:
        group_id:

    Returns:

    """
    return _add_or_remove_to_user_or_group_right_to_workspace(request, pk, group_id,
                                                              workspace_api.add_group_write_access_to_workspace,
                                                              group_api.get_group_by_id)


@api_view(['PATCH'])
def remove_user_read_right_to_workspace(request, pk, user_id):
    """ Remove from the user the read right to the workspace.

    Args:
        request:
        pk:
        user_id:

    Returns:

    """
    return _add_or_remove_to_user_or_group_right_to_workspace(request, pk, user_id,
                                                              workspace_api.remove_user_read_access_to_workspace,
                                                              user_api.get_user_by_id)


@api_view(['PATCH'])
def remove_user_write_right_to_workspace(request, pk, user_id):
    """ Remove from the user the write right to the workspace.

    Args:
        request:
        pk:
        user_id:

    Returns:

    """
    return _add_or_remove_to_user_or_group_right_to_workspace(request, pk, user_id,
                                                              workspace_api.remove_user_write_access_to_workspace,
                                                              user_api.get_user_by_id)


@api_view(['PATCH'])
def remove_group_read_right_to_workspace(request, pk, group_id):
    """ Remove from the group the read right to the workspace.

    Args:
        request:
        pk:
        group_id:

    Returns:

    """
    return _add_or_remove_to_user_or_group_right_to_workspace(request, pk, group_id,
                                                              workspace_api.remove_group_read_access_to_workspace,
                                                              group_api.get_group_by_id)


@api_view(['PATCH'])
def remove_group_write_right_to_workspace(request, pk, group_id):
    """ Remove from the group the write right to the workspace.

    Args:
        request:
        pk:
        group_id:

    Returns:

    """
    return _add_or_remove_to_user_or_group_right_to_workspace(request, pk, group_id,
                                                              workspace_api.remove_group_write_access_to_workspace,
                                                              group_api.get_group_by_id)





""" Access control testing
"""
import unittest

from mock.mock import patch

from core_main_app.components.data import api as data_api
from core_main_app.components.data.models import Data
from core_main_app.utils.access_control.exceptions import AccessControlError
from core_main_app.utils.integration_tests.integration_base_test_case import MongoIntegrationBaseTestCase
from core_main_app.utils.tests_tools.MockUser import create_mock_user
from tests.components.data.fixtures.fixtures import AccessControlDataFixture

fixture_data = AccessControlDataFixture()

# NOTE: Uncomment to test in debug
# from django.conf import settings
# settings.INSTALLED_APPS = ['core_workspace_app']


class TestDataGetById(MongoIntegrationBaseTestCase):

    fixture = fixture_data

    @patch('core_main_app.components.workspace.api.get_all_workspaces_with_read_access_by_user')
    def test_get_by_id_owner_with_read_access_returns_data(self, get_all_workspaces_with_read_access_by_user):
        data_id = self.fixture.data_collection[fixture_data.USER_1_WORKSPACE_1].id
        mock_user = _create_user('1')
        get_all_workspaces_with_read_access_by_user.return_value = [fixture_data.workspace_1]
        data = data_api.get_by_id(data_id, mock_user)
        self.assertTrue(isinstance(data, Data))

    @patch('core_main_app.components.workspace.api.get_all_workspaces_with_read_access_by_user')
    def test_get_by_id_owner_without_read_access_returns_data(self, get_all_workspaces_with_read_access_by_user):
        data_id = self.fixture.data_collection[fixture_data.USER_1_WORKSPACE_1].id
        mock_user = _create_user('1')
        get_all_workspaces_with_read_access_by_user.return_value = []
        with self.assertRaises(AccessControlError):
            data_api.get_by_id(data_id, mock_user)

    @patch('core_main_app.components.workspace.api.get_all_workspaces_with_read_access_by_user')
    def test_get_by_id_owner_without_read_access_returns_data(self, get_all_workspaces_with_read_access_by_user):
        data_id = self.fixture.data_collection[fixture_data.USER_1_WORKSPACE_1].id
        mock_user = _create_user('1')
        get_all_workspaces_with_read_access_by_user.return_value = []
        data = data_api.get_by_id(data_id, mock_user)
        self.assertTrue(isinstance(data, Data))

    @patch('core_main_app.components.workspace.api.get_all_workspaces_with_read_access_by_user')
    def test_get_by_id_user_without_read_access_raises_error(self, get_all_workspaces_with_read_access_by_user):
        data_id = self.fixture.data_collection[fixture_data.USER_1_WORKSPACE_1].id
        mock_user = _create_user('2')
        get_all_workspaces_with_read_access_by_user.return_value = []
        with self.assertRaises(AccessControlError):
            data_api.get_by_id(data_id, mock_user)

    def test_get_by_id_owner_no_workspace_read_access_returns_data(self):
        data_id = self.fixture.data_collection[fixture_data.USER_1_NO_WORKSPACE].id
        mock_user = _create_user('1')
        data = data_api.get_by_id(data_id, mock_user)
        self.assertTrue(isinstance(data, Data))

    def test_get_by_id_not_owner_no_workspace_raises_error(self):
        data_id = self.fixture.data_collection[fixture_data.USER_1_NO_WORKSPACE].id
        mock_user = _create_user('2')
        with self.assertRaises(AccessControlError):
            data_api.get_by_id(data_id, mock_user)


class TestDataGetAll(MongoIntegrationBaseTestCase):

    fixture = fixture_data

    def test_get_all_as_superuser_returns_all_data(self):
        mock_user = _create_user('1', is_superuser=True)
        data_list = data_api.get_all(mock_user)
        self.assertTrue(len(data_list) == len(self.fixture.data_collection))

    def test_get_all_as_user_raises_error(self):
        mock_user = _create_user('1')
        with self.assertRaises(AccessControlError):
            data_api.get_all(mock_user)


class TestDataGetAllByUser(MongoIntegrationBaseTestCase):

    fixture = fixture_data

    @patch('core_main_app.components.workspace.api.get_all_workspaces_with_read_access_by_user')
    def test_get_all_returns_data(self, get_all_workspaces_with_read_access_by_user):
        mock_user = _create_user('1')
        get_all_workspaces_with_read_access_by_user.return_value = []
        with self.assertRaises(AccessControlError):
            data_api.get_all_except_user(mock_user)

    @patch('core_main_app.components.workspace.api.get_all_workspaces_with_read_access_by_user')
    def test_get_all_by_user_returns_owned_data(self, get_all_workspaces_with_read_access_by_user):
        mock_user = _create_user('1')
        data_list = data_api.get_all_by_user(mock_user)
        get_all_workspaces_with_read_access_by_user.return_value = []
        self.assertTrue(len(data_list) == 2)
        self.assertTrue(data.id == '1' for data in data_list)

    @patch('core_main_app.components.workspace.api.get_all_workspaces_with_read_access_by_user')
    def test_get_all_by_user_returns_no_data_if_owns_zero(self, get_all_workspaces_with_read_access_by_user):
        mock_user = _create_user('3')
        data_list = data_api.get_all_by_user(mock_user)
        get_all_workspaces_with_read_access_by_user.return_value = []
        self.assertTrue(len(data_list) == 0)

    def test_get_all_by_user_as_superuser_returns_own_data(self):
        mock_user = _create_user('1', is_superuser=True)
        data_list = data_api.get_all_by_user(mock_user)
        self.assertTrue(len(data_list) == 2)
        self.assertTrue(data.user_id == '1' for data in data_list)


class TestDataGetAllExceptUser(MongoIntegrationBaseTestCase):
    # NOTE: Will always fail when private data are present (data.workspace=None, data.user_id!=user.id)
    fixture = fixture_data

    @patch('core_main_app.components.workspace.api.get_all_workspaces_with_read_access_by_user')
    def test_get_all_except_user_raises_error_if_no_workspace_access(self, get_all_workspaces_with_read_access_by_user):
        mock_user = _create_user('1')
        get_all_workspaces_with_read_access_by_user.return_value = []
        with self.assertRaises(AccessControlError):
            data_api.get_all_except_user(mock_user)

    @patch('core_main_app.components.workspace.api.get_all_workspaces_with_read_access_by_user')
    def test_get_all_except_user_raises_error_data_if_workspace_access(self, get_all_workspaces_with_read_access_by_user):
        mock_user = _create_user('1')
        get_all_workspaces_with_read_access_by_user.return_value = [fixture_data.workspace_1,
                                                                    fixture_data.workspace_2]
        with self.assertRaises(AccessControlError):
            data_api.get_all_except_user(mock_user)

    def test_get_all_except_user_as_superuser_returns_others_data(self):
        mock_user = _create_user('1', is_superuser=True)
        data_list = data_api.get_all_except_user(mock_user)
        self.assertTrue(len(data_list) > 0)
        self.assertTrue(data.user_id != mock_user.id for data in data_list)


class TestDataUpsert(MongoIntegrationBaseTestCase):
    # TODO: can not test without mock for GridFS
    pass


class TestDataExecuteQuery(MongoIntegrationBaseTestCase):

    fixture = fixture_data

    @patch('core_main_app.components.workspace.api.get_all_workspaces_with_read_access_by_user')
    def test_execute_query_returns_data(self, get_all_workspaces_with_read_access_by_user):
        mock_user = _create_user('3')
        get_all_workspaces_with_read_access_by_user.return_value = [fixture_data.workspace_1]
        data_list = data_api.execute_query({}, mock_user)
        self.assertTrue(len(data_list) > 0)
        self.assertTrue(all(isinstance(data, Data) for data in data_list))

    @patch('core_main_app.components.workspace.api.get_all_workspaces_with_read_access_by_user')
    def test_execute_query_returns_data_in_workspace_1(self, get_all_workspaces_with_read_access_by_user):
        mock_user = _create_user('3')
        get_all_workspaces_with_read_access_by_user.return_value = [fixture_data.workspace_1]
        data_list = data_api.execute_query({}, mock_user)
        self.assertTrue(len(data_list) == 1)
        self.assertTrue(data.workspace == '1' for data in data_list)

    @patch('core_main_app.components.workspace.api.get_all_workspaces_with_read_access_by_user')
    def test_execute_query_returns_data_in_workspace_2(self, get_all_workspaces_with_read_access_by_user):
        mock_user = _create_user('3')
        get_all_workspaces_with_read_access_by_user.return_value = [fixture_data.workspace_1]
        data_list = data_api.execute_query({}, mock_user)
        self.assertTrue(len(data_list) == 1)
        self.assertTrue(data.workspace == '2' for data in data_list)

    @patch('core_main_app.components.workspace.api.get_all_workspaces_with_read_access_by_user')
    def test_execute_query_returns_data_in_workspace_1_and_2(self, get_all_workspaces_with_read_access_by_user):
        mock_user = _create_user('3')
        get_all_workspaces_with_read_access_by_user.return_value = [fixture_data.workspace_1,
                                                                    fixture_data.workspace_2]
        data_list = data_api.execute_query({}, mock_user)
        self.assertTrue(len(data_list) == 2)
        self.assertTrue(data.workspace == '1' or data.workspace == '2' for data in data_list)

    @patch('core_main_app.components.workspace.api.get_all_workspaces_with_read_access_by_user')
    def test_execute_query_force_workspace_1_returns_data_from_workspace_1(self, get_all_workspaces_with_read_access_by_user):
        mock_user = _create_user('3')
        get_all_workspaces_with_read_access_by_user.return_value = [fixture_data.workspace_1]
        data_list = data_api.execute_query({'workspace': fixture_data.workspace_1.id}, mock_user)
        self.assertTrue(len(data_list) == 1)
        self.assertTrue(data.workspace == '1' for data in data_list)

    @patch('core_main_app.components.workspace.api.get_all_workspaces_with_read_access_by_user')
    def test_execute_query_force_workspace_1_does_not_return_data_if_no_access(self, get_all_workspaces_with_read_access_by_user):
        mock_user = _create_user('3')
        get_all_workspaces_with_read_access_by_user.return_value = []
        data_list = data_api.execute_query({'workspace': fixture_data.workspace_1.id}, mock_user)
        self.assertTrue(len(data_list) == 0)

    @patch('core_main_app.components.workspace.api.get_all_workspaces_with_read_access_by_user')
    def test_execute_query_force_workspace_none_does_not_return_data_if_no_access(self, get_all_workspaces_with_read_access_by_user):
        mock_user = _create_user('3')
        get_all_workspaces_with_read_access_by_user.return_value = []
        data_list = data_api.execute_query({'workspace': None}, mock_user)
        self.assertTrue(len(data_list) == 0)

    def test_execute_query_as_superuser_returns_all_data(self):
        mock_user = _create_user('1', is_superuser=True)
        data_list = data_api.execute_query({}, mock_user)
        self.assertTrue(len(data_list) == 4)


class TestDataDelete(MongoIntegrationBaseTestCase):

    fixture = fixture_data

    @unittest.skip("GridFS not supported by mongomock")
    @patch('core_main_app.components.workspace.api.get_all_workspaces_with_write_access_by_user')
    def test_delete_own_data_in_accessible_workspace_deletes_data(self, get_all_workspaces_with_write_access_by_user):
        mock_user = _create_user('1')
        get_all_workspaces_with_write_access_by_user.return_value = [fixture_data.workspace_1]
        data_api.delete(fixture_data.data_collection[fixture_data.USER_1_WORKSPACE_1], mock_user)

    @unittest.skip("GridFS not supported by mongomock")
    @patch('core_main_app.components.workspace.api.get_all_workspaces_with_write_access_by_user')
    def test_delete_own_data_in_not_accessible_workspace_deletes_data(self, get_all_workspaces_with_write_access_by_user):
        mock_user = _create_user('1')
        get_all_workspaces_with_write_access_by_user.return_value = []
        data_api.delete(fixture_data.data_collection[fixture_data.USER_1_WORKSPACE_1], mock_user)

    @unittest.skip("GridFS not supported by mongomock")
    @patch('core_main_app.components.workspace.api.get_all_workspaces_with_write_access_by_user')
    def test_delete_others_data_in_accessible_workspace_deletes_data(self, get_all_workspaces_with_write_access_by_user):
        mock_user = _create_user('1')
        get_all_workspaces_with_write_access_by_user.return_value = [fixture_data.workspace_2]
        data_api.delete(fixture_data.data_collection[fixture_data.USER_2_WORKSPACE_2], mock_user)

    @patch('core_main_app.components.workspace.api.get_all_workspaces_with_write_access_by_user')
    def test_delete_others_data_not_accessible_workspace_raises_error(self, get_all_workspaces_with_write_access_by_user):
        mock_user = _create_user('1')
        get_all_workspaces_with_write_access_by_user.return_value = [fixture_data.workspace_1]
        with self.assertRaises(AccessControlError):
            data_api.delete(fixture_data.data_collection[fixture_data.USER_2_WORKSPACE_2], mock_user)

    @unittest.skip("GridFS not supported by mongomock")
    @patch('core_main_app.components.workspace.api.get_all_workspaces_with_write_access_by_user')
    def test_delete_own_data_not_in_workspace_deletes_data(self, get_all_workspaces_with_write_access_by_user):
        mock_user = _create_user('1')
        get_all_workspaces_with_write_access_by_user.return_value = []
        with self.assertRaises(AccessControlError):
            data_api.delete(fixture_data.data_collection[fixture_data.USER_1_WORKSPACE_1], mock_user)

    @patch('core_main_app.components.workspace.api.get_all_workspaces_with_write_access_by_user')
    def test_delete_others_data_not_in_workspace_raises_error(self, get_all_workspaces_with_write_access_by_user):
        mock_user = _create_user('1')
        get_all_workspaces_with_write_access_by_user.return_value = []
        with self.assertRaises(AccessControlError):
            data_api.delete(fixture_data.data_collection[fixture_data.USER_2_NO_WORKSPACE], mock_user)


class TestDataChangeOwner(MongoIntegrationBaseTestCase):

    fixture = fixture_data

    def test_change_owner_from_owner_to_owner_ok(self):
        mock_owner = _create_user('1')
        data_api.change_owner(data=fixture_data.data_collection[fixture_data.USER_1_NO_WORKSPACE],
                              new_user=mock_owner,
                              user=mock_owner)

    def test_change_owner_from_owner_to_user_ok(self):
        mock_owner = _create_user('1')
        mock_user = _create_user('2')
        data_api.change_owner(data=fixture_data.data_collection[fixture_data.USER_1_NO_WORKSPACE],
                              new_user=mock_user,
                              user=mock_owner)

    def test_change_owner_from_user_to_user_raises_exception(self):
        mock_owner = _create_user('1')
        mock_user = _create_user('2')
        with self.assertRaises(AccessControlError):
            data_api.change_owner(data=fixture_data.data_collection[fixture_data.USER_1_NO_WORKSPACE],
                                  new_user=mock_owner,
                                  user=mock_user)

    def test_change_owner_as_superuser_ok(self):
        mock_user = _create_user('2', is_superuser=True)
        data_api.change_owner(data=fixture_data.data_collection[fixture_data.USER_1_NO_WORKSPACE],
                              new_user=mock_user,
                              user=mock_user)


def _create_user(user_id, is_superuser=False):
    return create_mock_user(user_id, is_superuser=is_superuser)



""" Fixtures files for User
"""

from django.contrib.auth.models import User

from core_main_app.utils.integration_tests.fixture_interface import FixtureInterface


class UserFixtures(FixtureInterface):
    """ User Fixture
    """

    def insert_data(self):
        pass

    @staticmethod
    def create_user(username="username", password=None, email=None):
        user = User.objects.create_user(username=username,
                                        password=password,
                                        email=email)
        user.save()
        return user

    @staticmethod
    def create_super_user(username="username", password="pass", email="email"):
        user = User.objects.create_superuser(username,
                                             password,
                                             email)
        user.save()
        return user